"""
Using the FORM - SORM algorithms on a nonlinear function
=========================================================
"""

# %%
# In this example, we estimate a failure probability with the `FORM` and `SORM` algorithms on the :ref:`oscillator <use-case-oscillator>` example.
# This test-case is highly non linear with a significant curvature near the design point.

# %%
# Model definition
# -----------------

# %%
from openturns.usecases import oscillator
import openturns as ot
from matplotlib import pyplot as plt
import numpy as np

ot.RandomGenerator.SetSeed(1)

# %%
# We load the model from the usecases module:
osc = oscillator.Oscillator()

# %%
# We use the input parameters distribution from the data class:
distribution = osc.distribution


# %%
# We define the model:
model = osc.model

# %%
# We create the event whose probability we want to estimate.

# %%
vect = ot.RandomVector(distribution)
G = ot.CompositeRandomVector(model, vect)
event = ot.ThresholdEvent(G, ot.Less(), 0.0)
event.setName("failure")

# %%
# Cross cuts in the physical space
# ---------------------------------

# %%
# Let’s have a look on 2D cross cuts of the limit state function. For each 2D cross cut, the other variables are fixed to the input distribution mean values.
# This graph allows one to have a first idea of the variations of the function in pairs of dimensions. The colors of each contour plot are comparable.
# The number of contour levels are related to the amount of variation of the function in the corresponding coordinates.


fig = plt.figure(figsize=(12, 12))
lowerBound = [1e-5] * 8
upperBound = distribution.getRange().getUpperBound()

# Definition of number of meshes in x and y axes for the 2D cross cut plots
nX = 50
nY = 50
for i in range(distribution.getDimension()):
    for j in range(i):
        crossCutIndices = []
        crossCutReferencePoint = []
        for k in range(distribution.getDimension()):
            if k != i and k != j:
                crossCutIndices.append(k)
                # Definition of the reference point
                crossCutReferencePoint.append(distribution.getMean()[k])
        # Definition of 2D cross cut function
        crossCutFunction = ot.ParametricFunction(
            model, crossCutIndices, crossCutReferencePoint
        )
        crossCutLowerBound = [lowerBound[j], lowerBound[i]]
        crossCutUpperBound = [upperBound[j], upperBound[i]]
        # Definition of the mesh
        inputData = ot.Box([nX, nY]).generate()
        inputData *= ot.Point(crossCutUpperBound) - ot.Point(crossCutLowerBound)
        inputData += ot.Point(crossCutLowerBound)
        meshX = np.array(inputData)[:, 0].reshape(nX + 2, nY + 2)
        meshY = np.array(inputData)[:, 1].reshape(nX + 2, nY + 2)
        data = crossCutFunction(inputData)
        meshZ = np.array(data).reshape(nX + 2, nY + 2)
        levels = [(150 + 3 * i) for i in range(101)]

        # Creation of the contour
        index = 1 + i * distribution.getDimension() + j

        ax = fig.add_subplot(
            distribution.getDimension(), distribution.getDimension(), index
        )
        ax.pcolormesh(meshX, meshY, meshZ, cmap="hsv", vmin=-5, vmax=50, shading="auto")
        ax.set_xticks([])
        ax.set_yticks([])
        # Creation of axes title
        if j == 0:
            ax.set_ylabel(distribution.getDescription()[i])
        if i == 7:
            ax.set_xlabel(distribution.getDescription()[j])

# %%
# Computation of reference probability with Monte-Carlo simulation
# ----------------------------------------------------------------

# %%
# The target probability is supposed to be extremely low (:math:`3.78\times 10^{-7}`).
# Indeed, when performing Monte-Carlo simulation with a simulation budget of 100000 points, no sample are in the failure state, that induces a probability estimate of zero.

experiment = ot.MonteCarloExperiment()
algo = ot.ProbabilitySimulationAlgorithm(event, experiment)
algo.setMaximumOuterSampling(int(1e5))
algo.run()

result = algo.getResult()
probability = result.getProbabilityEstimate()
print("Failure probability = ", probability)

# %%
# To get an accurate estimate of this probability, a billion of simulations are necessary.

# %%
# FORM Analysis
# -------------

# %%
# To get a first idea of the failure probability with reduced simulation budget, one can use the First Order Reliability Method (FORM).


# %%
# Define a solver:
optimAlgo = ot.Cobyla()
optimAlgo.setStartingPoint(distribution.getMean())
optimAlgo.setMaximumCallsNumber(1000)
optimAlgo.setMaximumAbsoluteError(1.0e-3)
optimAlgo.setMaximumRelativeError(1.0e-3)
optimAlgo.setMaximumResidualError(1.0e-3)
optimAlgo.setMaximumConstraintError(1.0e-3)

# %%
# Run FORM initialized at the mean of the distribution:
algo = ot.FORM(optimAlgo, event)
algo.run()
result = algo.getResult()

# %%
# Probability of failure:
print("FORM estimate of the probability of failure: ", result.getEventProbability())

# %%
# Design point in the standard space:

# %%
designPointStandardSpace = result.getStandardSpaceDesignPoint()
print("Design point in standard space : ", designPointStandardSpace)

# %%
# Design point in the physical space:

# %%
print("Design point in physical space : ", result.getPhysicalSpaceDesignPoint())


# %%
# For this test case, in order to validate the results obtained by FORM, we can plot the cross cuts in the standard space near the design point found by FORM.


# %%

distributionStandard = ot.Normal(distribution.getDimension())
inverseIsoProbabilistic = distribution.getInverseIsoProbabilisticTransformation()
standardSpaceLimitState = ot.ComposedFunction(model, inverseIsoProbabilistic)
standardSpaceLimitStateFunction = ot.PythonFunction(8, 1, standardSpaceLimitState)

fig = plt.figure(figsize=(12, 12))
lowerBound = [-4] * 8
upperBound = [4] * 8

# sphinx_gallery_thumbnail_number = 2

# Definition of number of meshes in x and y axes for the 2D cross cut plots
nX = 50
nY = 50
my_labels = {
    "MPP": "Design Point",
    "O": "Origin in Standard Space",
    "TLSF": "True Limit State Function",
    "ALSF": "Approximated Limit State Function",
}
for i in range(distribution.getDimension()):
    for j in range(i):
        crossCutIndices = []
        crossCutReferencePoint = []
        for k in range(distribution.getDimension()):
            if k != i and k != j:
                crossCutIndices.append(k)
                # Definition of the reference point
                crossCutReferencePoint.append(designPointStandardSpace[k])
        # Definition of 2D cross cut function
        crossCutFunction = ot.ParametricFunction(
            standardSpaceLimitStateFunction, crossCutIndices, crossCutReferencePoint
        )

        crossCutLowerBound = [
            lowerBound[j] + designPointStandardSpace[j],
            lowerBound[i] + designPointStandardSpace[i],
        ]
        crossCutUpperBound = [
            upperBound[j] + designPointStandardSpace[j],
            upperBound[i] + designPointStandardSpace[i],
        ]
        # Definition of the mesh
        inputData = ot.Box([nX, nY]).generate()
        inputData *= ot.Point(crossCutUpperBound) - ot.Point(crossCutLowerBound)
        inputData += ot.Point(crossCutLowerBound)
        meshX = np.array(inputData)[:, 0].reshape(nX + 2, nY + 2)
        meshY = np.array(inputData)[:, 1].reshape(nX + 2, nY + 2)
        data = crossCutFunction(inputData)
        meshZ = np.array(data).reshape(nX + 2, nY + 2)
        levels = [(150 + 3 * i) for i in range(101)]

        # Creation of the contour
        index = 1 + i * distribution.getDimension() + j

        ax = fig.add_subplot(
            distribution.getDimension(), distribution.getDimension(), index
        )

        graph = ot.Graph()

        ax.pcolormesh(meshX, meshY, meshZ, cmap="hsv", vmin=-5, vmax=50, shading="auto")

        cs0 = ax.plot(
            designPointStandardSpace[j],
            designPointStandardSpace[i],
            "o",
            label=my_labels["MPP"],
        )
        cs1 = ax.plot(0.0, 0.0, "rs", label=my_labels["O"])
        cs2 = ax.contour(meshX, meshY, meshZ, [0.0])

        ax.set_xticks([])
        ax.set_yticks([])

        u0 = [designPointStandardSpace[j], designPointStandardSpace[i]]
        algo = ot.LinearTaylor(u0, crossCutFunction)
        algo.run()
        responseSurface = algo.getMetaModel()
        data2 = responseSurface(inputData)
        meshZ2 = np.array(data2).reshape(nX + 2, nY + 2)

        cs3 = ax.contour(meshX, meshY, meshZ2, [0.0], linestyles="dotted")

        # Creation of axes title
        if j == 0:
            ax.set_ylabel(distribution.getDescription()[i])
        if i == 7:
            ax.set_xlabel(distribution.getDescription()[j])

        if i == 1 and j == 0:
            h2, l2 = cs2.legend_elements()
            h3, l3 = cs3.legend_elements()
            lg = ax.legend(
                [h2[0], h3[0]],
                [my_labels["TLSF"], my_labels["ALSF"]],
                frameon=False,
                loc="upper center",
                bbox_to_anchor=(8, -1.5),
            )
        elif i == 2 and j == 0:
            lg1 = ax.legend(
                frameon=False, loc="upper center", bbox_to_anchor=(7.65, -0.8)
            )

# %%
# As it can be seen, the curvature of the limit state function near the design point is significant.
# In that way, FORM provides poor estimate since it linearly approximates the limit state function.
# Thus, SORM can be used in order to refine this probability estimation by approximating the limit state function with a quadratic model.

# %%
# SORM Analysis
# -------------

# %%
# Run SORM initialized at the mean of the distribution:


# %%
solver = ot.Cobyla()
solver.setStartingPoint(distribution.getMean())
algoSORM = ot.SORM(solver, event)
algoSORM.run()

# %%
# Analysis of SORM results:

# %%
resultSORM = algoSORM.getResult()
print(
    "Probability estimate with SORM (Breitung correction) =",
    resultSORM.getEventProbabilityBreitung(),
)

print(
    "Probability estimate with SORM (Hohenbichler correction) =",
    resultSORM.getEventProbabilityHohenbichler(),
)

optim_res = resultSORM.getOptimizationResult()
print("Simulation budget:", optim_res.getCallsNumber())

# %%
# One can see that the probability estimate has been decreased by a factor 10 compared to the FORM estimate.
# This probability is quite close to the reference probability and obtained with less than 1000 evaluations of the model.

# %%
# In order to visualize the SORM limit state approximation, we can draw cross cuts of SORM oscultating parabola using second order Taylor approximation.

fig = plt.figure(figsize=(12, 12))
lowerBound = [-4] * 8
upperBound = [4] * 8

# Definition of number of meshes in x and y axes for the 2D cross cut plots
nX = 50
nY = 50

my_labels = {
    "MPP": "Design Point",
    "O": "Origin in Standard Space",
    "TLSF": "True Limit State Function",
    "ALSF": "Approximated Limit State Function",
}

for i in range(distribution.getDimension()):
    for j in range(i):
        crossCutIndices = []
        crossCutReferencePoint = []
        for k in range(distribution.getDimension()):
            if k != i and k != j:
                crossCutIndices.append(k)
                # Definition of the reference point
                crossCutReferencePoint.append(designPointStandardSpace[k])
        # Definition of 2D cross cut function
        crossCutFunction = ot.ParametricFunction(
            standardSpaceLimitStateFunction, crossCutIndices, crossCutReferencePoint
        )

        crossCutLowerBound = [
            lowerBound[j] + designPointStandardSpace[j],
            lowerBound[i] + designPointStandardSpace[i],
        ]
        crossCutUpperBound = [
            upperBound[j] + designPointStandardSpace[j],
            upperBound[i] + designPointStandardSpace[i],
        ]
        # Definition of the mesh
        inputData = ot.Box([nX, nY]).generate()
        inputData *= ot.Point(crossCutUpperBound) - ot.Point(crossCutLowerBound)
        inputData += ot.Point(crossCutLowerBound)
        meshX = np.array(inputData)[:, 0].reshape(nX + 2, nY + 2)
        meshY = np.array(inputData)[:, 1].reshape(nX + 2, nY + 2)
        data = crossCutFunction(inputData)
        meshZ = np.array(data).reshape(nX + 2, nY + 2)
        levels = [(150 + 3 * i) for i in range(101)]

        # Creation of the contour
        index = 1 + i * distribution.getDimension() + j

        ax = fig.add_subplot(
            distribution.getDimension(), distribution.getDimension(), index
        )

        graph = ot.Graph()
        ax.pcolormesh(meshX, meshY, meshZ, cmap="hsv", vmin=-5, vmax=50, shading="auto")
        cs0 = ax.plot(
            designPointStandardSpace[j],
            designPointStandardSpace[i],
            "o",
            label=my_labels["MPP"],
        )
        cs1 = ax.plot(0.0, 0.0, "rs", label=my_labels["O"])
        cs2 = ax.contour(meshX, meshY, meshZ, [0.0])
        ax.set_xticks([])
        ax.set_yticks([])

        u0 = [designPointStandardSpace[j], designPointStandardSpace[i]]
        algo = ot.QuadraticTaylor(u0, crossCutFunction)
        algo.run()
        responseSurface = algo.getMetaModel()
        data2 = responseSurface(inputData)
        meshZ2 = np.array(data2).reshape(nX + 2, nY + 2)
        cs3 = ax.contour(meshX, meshY, meshZ2, [0.0], linestyles="dotted")

        # Creation of axes title
        if j == 0:
            ax.set_ylabel(distribution.getDescription()[i])
        if i == 7:
            ax.set_xlabel(distribution.getDescription()[j])

        if i == 1 and j == 0:
            h2, l2 = cs2.legend_elements()
            h3, l3 = cs3.legend_elements()
            lg = ax.legend(
                [h2[0], h3[0]],
                [my_labels["TLSF"], my_labels["ALSF"]],
                frameon=False,
                loc="upper center",
                bbox_to_anchor=(8, -1.5),
            )
        elif i == 2 and j == 0:
            lg1 = ax.legend(
                frameon=False, loc="upper center", bbox_to_anchor=(7.65, -0.8)
            )
# %%
# We can see that this approximation is very appropriate, explaining the accuracy of the obtained results.

# %%
# Estimation with post analytical Importance Sampling
# ---------------------------------------------------

# %%
# Different algorithms exist for the reliability analysis by Importance Sampling.
# One way is to perform post analytical Importance Sampling by defining the auxiliary density centered at the design point found by FORM.


# %%
postAnalyticalIS = ot.PostAnalyticalImportanceSampling(result)
postAnalyticalIS.setMaximumCoefficientOfVariation(0.05)
postAnalyticalIS.run()
resultPostAnalyticalIS = postAnalyticalIS.getResult()
print(
    "Probability of failure with post analytical IS = ",
    resultPostAnalyticalIS.getProbabilityEstimate(),
)

# %%
# We can see that this post-treatment of FORM result allows one to greatly improve the quality of the probability estimation.



#! /usr/bin/env python

import copy
import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()
mesh = ot.RegularGrid(0, 1, 4)
vals1 = ot.Sample([[1, 2, 3], [2, 3, 4], [3, 4, 5], [4, 5, 6]])
vals2 = ot.Sample([[0, 1, 2], [1, 2, 3], [2, 3, 4], [3, 4, 5]])

f1 = ot.Field(mesh, vals1)
f2 = ot.Field(mesh, vals2)

s = vals1
p = ot.Point((1, 2, 3))

# Field operations
fSum = ot.Field(mesh, vals1 + vals2)
fDiff = ot.Field(mesh, vals1 - vals2)

ott.assert_almost_equal(f1 + f2, fSum)
ott.assert_almost_equal(f1 - f2, fDiff)
fInPlaceSum = copy.deepcopy(f1)
fInPlaceSum += f2
ott.assert_almost_equal(fInPlaceSum, fSum)
fInPlaceSum -= f2
ott.assert_almost_equal(fInPlaceSum, f1)

# Field and Sample operations
fSum = ot.Field(mesh, vals1 + vals1)
fDiff = ot.Field(mesh, vals1 - vals1)

ott.assert_almost_equal(f1 + s, fSum)
ott.assert_almost_equal(f1 - s, fDiff)
fInPlaceSum = copy.deepcopy(f1)
fInPlaceSum += s
ott.assert_almost_equal(fInPlaceSum, fSum)
fInPlaceSum -= s
ott.assert_almost_equal(fInPlaceSum, f1)

# Field and Point operations
fSum = ot.Field(mesh, vals1 + p)
fDiff = ot.Field(mesh, vals1 - p)

ott.assert_almost_equal(f1 + p, fSum)
ott.assert_almost_equal(f1 - p, fDiff)
fInPlaceSum = copy.deepcopy(f1)
fInPlaceSum += p
ott.assert_almost_equal(fInPlaceSum, fSum)
fInPlaceSum -= p
ott.assert_almost_equal(fInPlaceSum, f1)



import openturns as ot
import openturns.viewer as otv

f = ot.SymbolicFunction(["x"], ["sin(pi_*sin(pi_*x))"])
graph = ot.Graph(r"Function $x  \mapsto \sin(\pi \sin(\pi x))$", r"$x$", r"$y$")
graph.add(f.draw(-0.5, 3.0, 1024))

view = otv.View(graph, figure_kw={"figsize": (10, 4)}, add_legend=False)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

dim = 10
R = ot.CorrelationMatrix(dim)
for i in range(dim):
    for j in range(i):
        R[i, j] = (i + j + 1.0) / (2.0 * dim)
mean = ot.Point(dim, 2.0)
sigma = ot.Point(dim, 3.0)
distribution = ot.Normal(mean, sigma, R)

size = 100
sample = distribution.getSample(size)
sampleX = ot.Sample(size, dim - 1)
sampleY = ot.Sample(size, 1)
for i in range(size):
    sampleY[i, 0] = sample[i, 0]
    for j in range(1, dim):
        sampleX[i, j - 1] = sample[i, j]

sampleZ = ot.Sample(size, 1)
for i in range(size):
    sampleZ[i, 0] = sampleY[i, 0] * sampleY[i, 0]

selection = ot.Indices(5)
selection.fill()
print("selection = ", selection)

# PartialPearson : Independence Pearson test between 2 samples : firstSample of dimension n and secondSample of dimension 1.
# If firstSample[i] is the numerical sample extracted from firstSample (ith coordinate of each point of the numerical sample),
# PartialPearson performs the Independence Pearson test simultaneously on firstSample[i] and secondSample, for i in the selection.
# For all i, it is supposed that the couple (firstSample[i] and secondSample) is issued from a gaussian  vector.
# Probability of the H0 reject zone : 0.10

print(
    "PartialPearsonXY=",
    ot.HypothesisTest.PartialPearson(sampleX, sampleY, selection, 0.10),
)

selection2 = ot.Indices(1, 0)
sampleX0 = ot.Sample(size, 1)
for i in range(size):
    sampleX0[i, 0] = sampleX[i, 0]

# The three tests must be equal
print(
    "PartialPearsonX0Y=",
    ot.HypothesisTest.PartialPearson(sampleX, sampleY, selection2, 0.10),
)
print("PearsonX0Y=", ot.HypothesisTest.Pearson(sampleX0, sampleY, 0.10))
print("FullPearsonX0Y=", ot.HypothesisTest.FullPearson(sampleX0, sampleY, 0.10))

# FullPearson : Independence Pearson test between 2 samples : firstSample of dimension n and secondSample of dimension 1.
# If firstSample[i] is the numerical sample extracted from firstSample (ith coordinate of each point of the numerical sample),
# FullPearson performs the Independence Pearson test simultaneously on firstSample[i] and secondSample.
# For all i, it is supposed that the couple (firstSample[i] and secondSample) is issued from a gaussian  vector.
# Probability of the H0 reject zone : 0.10
print("FullPearsonXY=", ot.HypothesisTest.FullPearson(sampleX, sampleY, 0.10))

# Expected result is SpecFunc::Infinity
fullPearsonYY = ot.HypothesisTest.FullPearson(sampleY, sampleY, 0.10)[0]
testStatistic = fullPearsonYY.getStatistic()
assert testStatistic == ot.SpecFunc.Infinity



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

distribution = ot.Binomial(15, 0.7)
size = 10000
sample = distribution.getSample(size)
factory = ot.BinomialFactory()
estimatedDistribution = factory.build(sample)
print("distribution=", repr(distribution))
print("Estimated distribution=", repr(estimatedDistribution))
estimatedDistribution = factory.build()
print("Default distribution=", repr(estimatedDistribution))
estimatedDistribution = factory.build(distribution.getParameter())
print("Distribution from parameters=", repr(estimatedDistribution))
estimatedBinomial = factory.buildAsBinomial(sample)
print("Binomial          =", distribution)
print("Estimated binomial=", estimatedBinomial)
estimatedBinomial = factory.buildAsBinomial()
print("Default binomial=", estimatedBinomial)
estimatedBinomial = factory.buildAsBinomial(distribution.getParameter())
print("Binomial from parameters=", estimatedBinomial)



"""
Create and draw scalar distributions
====================================
"""

# %%
import openturns as ot
import openturns.viewer as otv

# %%
# A continuous distribution
# -------------------------
#
# We build a Normal distribution with parameters:
#
# .. math::
#     \mu = 2.2, \sigma = 0.6
#
distribution = ot.Normal(2.2, 0.6)
print(distribution)

# %%
# We can draw a sample following this distribution with the `getSample` method :
size = 10
sample = distribution.getSample(size)
print(sample)

# %%
# We draw its PDF and CDF :
graphPDF = distribution.drawPDF()
graphPDF.setTitle(
    r"PDF of a normal distribution with parameters $\mu = 2.2$ and $\sigma = 0.6$"
)
view = otv.View(graphPDF)

# %%
graphCDF = distribution.drawCDF()
graphCDF.setTitle(
    r"CDF of a normal distribution with parameters $\mu = 2.2$ and $\sigma = 0.6$"
)
view = otv.View(graphCDF)


# %%
# A discrete distribution
# -----------------------
#
# We define a geometric distribution with parameter :math:`p = 0.7`.
p = 0.7
distribution = ot.Geometric(p)
print(distribution)

# %%
# We draw a sample of it :
size = 10
sample = distribution.getSample(size)
print(sample)

# %%
# We draw its PDF and its CDF :
graphPDF = distribution.drawPDF()
graphPDF.setTitle(r"PDF of a geometric distribution with parameter $p = 0.7$")
view = otv.View(graphPDF)

# %%
graphCDF = distribution.drawCDF()
graphCDF.setTitle(r"CDF of a geometric distribution with parameter $p = 0.7$")
view = otv.View(graphCDF)

# %%
# Conclusion
# ----------
#
# The two previous examples look very similar despite their continuous and discrete nature.
# In the library there is no distinction between continuous and discrete distributions.

# %%
# Display all figures
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot
from openturns.usecases import ishigami_function

ot.TESTPREAMBLE()

# Problem parameters
im = ishigami_function.IshigamiModel()
distribution = im.distribution
model = im.model
dimension = im.distribution.getDimension()
sobolFirstOrderReference = [im.S1, im.S2, im.S3]
sobolSecondOrderReference = [im.S12, im.S13, im.S23]
sobolTotalReference = [im.ST1, im.ST2, im.ST3]
sobolInteractionTotalReference2 = [im.S12 + im.S123, im.S13 + im.S123, im.S23 + im.S123]
sobolInteractionTotalReference3 = [im.S123]
sobolGroupedIndex2 = [
    im.S1 + im.S2 + im.S12,
    im.S1 + im.S3 + im.S13,
    im.S2 + im.S3 + im.S23,
]
sobolGroupedIndex3 = [im.S1 + im.S2 + im.S3 + im.S12 + im.S13 + im.S23 + im.S123]

# Create the orthogonal basis
enumerateFunction = ot.LinearEnumerateFunction(dimension)
productBasis = ot.OrthogonalProductPolynomialFactory(
    [ot.LegendreFactory()] * dimension, enumerateFunction
)

# Create the adaptive strategy
# We can choose amongst several strategies
# First, the most efficient (but more complex!) strategy
listAdaptiveStrategy = list()
degree = 6
indexMax = enumerateFunction.getBasisSizeFromTotalDegree(degree)
basisDimension = enumerateFunction.getBasisSizeFromTotalDegree(degree // 2)
threshold = 1.0e-6
listAdaptiveStrategy.append(
    ot.CleaningStrategy(productBasis, indexMax, basisDimension, threshold)
)
# Second, the most used (and most basic!) strategy
listAdaptiveStrategy.append(
    ot.FixedStrategy(
        productBasis, enumerateFunction.getBasisSizeFromTotalDegree(degree)
    )
)

for adaptiveStrategyIndex in range(len(listAdaptiveStrategy)):
    adaptiveStrategy = listAdaptiveStrategy[adaptiveStrategyIndex]
    # Create the projection strategy
    samplingSize = 250
    listExperiment = list()
    # We have only the LeastSquaresStrategy up to now (0.13.0) but we can choose several sampling schemes
    # Monte Carlo sampling
    listExperiment.append(ot.MonteCarloExperiment(distribution, samplingSize))
    # LHS sampling
    listExperiment.append(ot.LHSExperiment(distribution, samplingSize))
    # Low Discrepancy sequence
    listExperiment.append(
        ot.LowDiscrepancyExperiment(ot.SobolSequence(), distribution, samplingSize)
    )
    for experiment in listExperiment:
        # Create the polynomial chaos algorithm
        maximumResidual = 1.0e-10
        ot.RandomGenerator.SetSeed(0)
        X = experiment.generate()
        Y = model(X)
        algo = ot.FunctionalChaosAlgorithm(X, Y, distribution, adaptiveStrategy)
        algo.setMaximumResidual(maximumResidual)
        algo.run()

        # Examine the results
        result = algo.getResult()
        pGrad = result.getMetaModel().parameterGradient(distribution.getMean())
        print("###################################")
        print(algo.getAdaptiveStrategy())
        print(algo.getProjectionStrategy())
        # print "coefficients=", result.getCoefficients()
        isLeastSquaresPCE = result.isLeastSquares()
        assert isLeastSquaresPCE
        involvesModelSelectionPCE = result.involvesModelSelection()
        involvesModelSelection = adaptiveStrategy.involvesModelSelection()
        assert involvesModelSelection == involvesModelSelectionPCE

        # Post-process the results
        vector = ot.FunctionalChaosRandomVector(result)
        mean = vector.getMean()[0]
        print("mean=%.8f" % mean, "absolute error=%.10f" % abs(mean - im.expectation))
        variance = vector.getCovariance()[0, 0]
        print(
            "variance=%.8f" % variance,
            "absolute error=%.10f" % abs(variance - im.variance),
        )
        sensitivity = ot.FunctionalChaosSobolIndices(result)
        for i in range(dimension):
            value = sensitivity.getSobolIndex(i)
            print(
                "Sobol index",
                i,
                "= %.8f" % value,
                "absolute error=%.10f" % abs(value - sobolFirstOrderReference[i]),
            )
        indices = ot.Indices(2)
        k = 0
        for i in range(dimension):
            indices[0] = i
            for j in range(i + 1, dimension):
                indices[1] = j
                value = sensitivity.getSobolIndex(indices)
                print(
                    "Sobol index",
                    indices,
                    "=%.8f" % value,
                    "absolute error=%.10f" % abs(value - sobolSecondOrderReference[k]),
                )
                k = k + 1
        indices = ot.Indices([0, 1, 2])
        value = sensitivity.getSobolIndex(indices)
        print(
            "Sobol index",
            indices,
            "=%.8f" % value,
            "absolute error=%.10f" % abs(value - im.S123),
        )
        for i in range(dimension):
            value = sensitivity.getSobolTotalIndex(i)
            print(
                "Sobol total index",
                i,
                "=%.8f" % value,
                "absolute error=%.10f" % abs(value - sobolTotalReference[i]),
            )
        indices = ot.Indices(2)
        k = 0
        for i in range(dimension):
            indices[0] = i
            for j in range(i + 1, dimension):
                indices[1] = j
                value = sensitivity.getSobolTotalIndex(indices)
                print(
                    "Sobol total index",
                    indices,
                    "=%.8f" % value,
                    "absolute error=%.10f"
                    % abs(value - sobolInteractionTotalReference2[k]),
                )
                k = k + 1
        indices = ot.Indices([0, 1, 2])
        value = sensitivity.getSobolTotalIndex(indices)
        print(
            "Sobol total index ",
            indices,
            "=%.8f" % value,
            "absolute error=%.10f" % abs(value - sobolInteractionTotalReference3[0]),
        )
        indices = ot.Indices(2)
        k = 0
        for i in range(dimension):
            indices[0] = i
            for j in range(i + 1, dimension):
                indices[1] = j
                value = sensitivity.getSobolGroupedIndex(indices)
                print(
                    "Sobol grouped index",
                    indices,
                    "=%.8f" % value,
                    "absolute error=%.10f" % abs(value - sobolGroupedIndex2[k]),
                )
                k = k + 1
        indices = ot.Indices([0, 1, 2])
        value = sensitivity.getSobolGroupedIndex(indices)
        print(
            "Sobol grouped index ",
            indices,
            "=%.8f" % value,
            "absolute error=%.10f" % abs(value - sobolGroupedIndex3[0]),
        )
        indices = ot.Indices([0, 1, 2])
        value = sensitivity.getSobolGroupedTotalIndex(indices)
        print(
            "Sobol grouped total index ",
            indices,
            "=%.8f" % value,
            "absolute error=%.10f" % abs(value - 1.0),
        )
        # Get part of variance indices
        print("Part of variance")
        partOfVariance = sensitivity.getPartOfVariance()
        result = sensitivity.getFunctionalChaosResult()
        orthogonalBasis = result.getOrthogonalBasis()
        enumerateFunction = orthogonalBasis.getEnumerateFunction()
        indices = result.getIndices()
        basisSize = indices.getSize()
        for i in range(basisSize):
            globalIndex = indices[i]
            multiIndex = enumerateFunction(globalIndex)
            if partOfVariance[i] > 1.0e-3:
                print(
                    "%d, %d, %s, %.4f" % (i, globalIndex, multiIndex, partOfVariance[i])
                )
        # Print summary
        print(sensitivity)



#! /usr/bin/env python

import openturns as ot


# We create an empty sample
N = 10
d = 5
sample = ot.Sample(N, d)
print("sample[-1]", sample[-1])
sample[-1] = [-1.0] * d
print("sample[-1]=-1", sample[-1])

# We populate the empty sample with numerous data
for i in range(N):
    for j in range(d):
        sample[i, j] = 100.0 * (j + 1) + i
description = []
for j in range(d):
    description.append("V" + str(j + 1))
sample.setDescription(description)
print("sample=", str(sample))


# case 0.1 [slice]
print("rslice[:]=", sample[:])
print("rslice[2:4]=", sample[2:4])
print("rslice[1:8:2]=", sample[1:8:2])

# case 0.2 [sequence]
print("rslice[[4,2,2,-1]]=", sample[[4, 2, 2, -1]])

# case 1.1 [int/int]
print("rslice[3,4]=", sample[3, 4])

# case 1.2 [int/slice]
print("rslice[3,:]=", sample[3, :])
print("rslice[3,1:4:2]=", sample[3, 1:4:2])

# case 1.3 [int/sequence]
print("rslice[3, (4,2,2,-1)]=", sample[3, (4, 2, 2, -1)])

# case 2.1 [slice/int]
print("rslice[:,3]=", sample[:, 3])
print("rslice[2:7:2,3]=", sample[2:7:2, 3])

# case 2.2 [slice/slice]
print("rslice[:,:]=", sample[:, :])
print("rslice[2:4,2:4]=", sample[2:4, 2:4])
print("rslice[1:8:2,1:5:3]=", sample[1:8:2, 1:5:3])

# case 2.3 [slice/sequence]
print("rslice[:, (4,2,2,-1)]=", sample[:, (4, 2, 2, -1)])

# case 3.1 [sequence/int]
print("rslice[(4,2,2,-1), 3]=", sample[(4, 2, 2, -1), 3])

# case 3.2 [sequence/slice]
print("rslice[(4,2,2,-1), :]=", sample[(4, 2, 2, -1), :])

# case 3.3 [sequence/sequence]
print("rslice[(4,2,2,-1), (3,1,-1)]=", sample[(4, 2, 2, -1), (3, 1, -1)])

# Writing slice
sample2 = ot.Sample(sample)
sample2 *= [10.0] * d

# case 0.1 [slice]
sample3 = ot.Sample(sample)
sample3[:] = sample2
print("wrslice[:]=", sample3)
sample3 = ot.Sample(sample)
sample3[2:4] = sample2[2:4]
print("wslice[2:4]=", sample3)
sample3 = ot.Sample(sample)
sample3[1:8:2] = sample2[1:8:2]
print("wslice[1:8:2]=", sample3)

# case 0.2 [sequence]
sample3 = ot.Sample(sample)
sample3[[4, 2, 2, -1]] = sample2[[4, 2, 2, -1]]
print("wslice[[4,2,2,-1]]=", sample3)

# case 1.1 [int/int]
sample3 = ot.Sample(sample)
sample3[3, 4] = sample2[3, 4]
print("wslice[3,4]=", sample3)

# case 1.2 [int/slice]
sample3 = ot.Sample(sample)
sample3[3, :] = sample2[3, :]
print("wslice[3,:]=", sample3)
sample3 = ot.Sample(sample)
sample3[3, 1:4:2] = sample2[3, 1:4:2]
print("wslice[3,1:4:2]=", sample3)

# case 1.3 [int/sequence]
sample3 = ot.Sample(sample)
sample3[3, [4, 2, 2, -1]] = sample2[3, [4, 2, 2, -1]]
print("wslice[3, [4,2,2,-1]]=", sample3)

# case 2.1 [slice/int]
sample3 = ot.Sample(sample)
sample3[:, 3] = sample2[:, 3]
print("wslice[:,3]=", sample3)
sample3 = ot.Sample(sample)
sample3[2:7:2, 3] = sample2[2:7:2, 3]
print("wslice[2:7:2,3]=", sample3)

# case 2.2 [slice/slice]
sample3 = ot.Sample(sample)
sample3[:, :] = sample2
print("wrslice[:,:]=", sample3)
sample3 = ot.Sample(sample)
sample3[2:4, 2:4] = sample2[2:4, 2:4]
print("wslice[2:4,2:4]=", sample3)
sample3 = ot.Sample(sample)
sample3[1:8:2, 1:5:3] = sample2[1:8:2, 1:5:3]
print("wslice[1:8:2,1:5:3]=", sample3)

# case 3.1 [sequence/int]
sample3 = ot.Sample(sample)
sample3[[4, 2, 2, -1], 3] = sample2[[4, 2, 2, -1], 3]
print("wslice[[4,2,2,-1], 3]=", sample3)



import openturns as ot
import openturns.viewer as otv
import openturns.experimental as otexp

func = ot.SymbolicFunction(
    ["x1", "x2", "x3"], ["x1 + x2 + sin(x2 * 2 * pi_) / 5 + 1e-3 * x3^2"]
)

dimension = 3

distribution = ot.JointDistribution([ot.Normal()] * dimension)
sampleSize = 50
inputSample = distribution.getSample(sampleSize)
outputSample = func(inputSample)
algo = ot.LinearModelAlgorithm(inputSample, outputSample)
algo.run()
result = algo.getResult()
splitter = ot.LeaveOneOutSplitter(sampleSize)
validation = otexp.LinearModelValidation(result, splitter)
graph = validation.drawValidation()
otv.View(graph)



"""
Create and manipulate an ARMA process
=====================================
"""

# %%
# In this example we first create an ARMA process and then manipulate it.
#

# %%
import openturns as ot
import openturns.viewer as otv

# %%
# Create an ARMA process
# ----------------------
#
# In this example we are going to build an ARMA process defined by its linear recurrence coefficients.
#
# The creation of an ARMA model requires the data of the AR and MA
# coefficients which are:
#
# -  a list of scalars in the unidmensional case :
#    :math:`(a_1, \dots, a_p)` for the AR-coefficients and
#    :math:`(b_1, \dots, b_q)` for the MA-coefficients
#
# -  a list of square matrix
#    :math:`(\mat{A}_{\, 1}, \dots, \mat{A}{\, _p})` for the
#    AR-coefficients and
#    :math:`(\mat{B}_{\, 1}\, \dots, \mat{B}_{\, q})` for the
#    MA-coefficients
#
# It also requires the definition of a white noise
# :math:`\vect{\varepsilon}` that contains the same time grid as the
# one of the process.
# The current state of an ARMA model is characterized by its last
# :math:`p` values and the last :math:`q` values of its white noise. It
# is possible to get that state thanks to the methods `getState`.
# It is possible to create an ARMA with a specific current state. That
# specific current state is taken into account to generate possible
# futures but not to generate realizations (in order to respect the
# stationarity property of the model).
# At the creation step, we check whether the process
# :math:`ARMA(p,q)` is stationnary.
# When the process is not stationary, the user is warned by a message.

# %%
# We build the 1-d ARMA process defined by:
#
# .. math::
#    X_{0,t} + 0.4 X_{0,t-1} + 0.3 X_{0,t-2} + 0.2 X_{0,t-3} + 0.1 X_{0,t-4} = E_{0,t} + 0.4 E_{0,t-1} + 0.3 E_{0,t-2}
#
# where the white noise :math:`E_t` is defined by :math:`E_t \approx \mathrm{Triangular}(a = -1, m = 0, b = 1)`.
#

# %%
# The definition of the recurrence coefficients ARMA (4,2) is simple:
myARCoef = ot.ARMACoefficients([0.4, 0.3, 0.2, 0.1])
myMACoef = ot.ARMACoefficients([0.4, 0.3])


# %%
# We build a regular time discretization of the interval [0,1] with 10 time steps.
# We also set up the white noise distribution of the recurrence relation :
myTimeGrid = ot.RegularGrid(0.0, 0.1, 10)
myWhiteNoise = ot.WhiteNoise(ot.Triangular(-1.0, 0.0, 1.0), myTimeGrid)

# %%
# We are now ready to create the ARMA-process :
process = ot.ARMA(myARCoef, myMACoef, myWhiteNoise)
print(process)


# %%
# ARMA process manipulation
# -------------------------
#
# In this paragraph we shall expose some of the services exposed by an :math:`ARMA(p,q)` object, namely :
#
# -  its AR and MA coefficients thanks to the methods `getARCoefficients`,
#    `getMACoefficients`,
#
# -  its white noise thanks to the method `getWhiteNoise`, that contains
#    the time grid of the process,
#
# -  its current state, that is its last :math:`p` values and the last
#    :math:`q` values of its white noise, thanks to the method `getState`,
#
# -  a realization thanks to the method `getRealization` or a sample of realizations thanks to the method `getSample`,
#
# -  a possible future of the model, which is a possible extension of
#    the current state on the next :math:`n_{prol}` instants, thanks to
#    the method `getFuture`.
#
# -  :math:`n` possible futures of the model, which correspond to
#    :math:`n` possible prolongations of the current state on the next
#    :math:`n_{prol}` instants, thanks to the method
#    `getFuture` (:math:`n_{prol}`, :math:`n`).
#


# %%
# First we get the coefficients AR and MA of the recurrence :
print("AR coeff = ", process.getARCoefficients())
print("MA coeff = ", process.getMACoefficients())

# %%
# We check that the white noise is the one we have previously defined :
myWhiteNoise = process.getWhiteNoise()
print(myWhiteNoise)


# %%
# We generate a possible time series realization :
ts = process.getRealization()
ts.setName("ARMA realization")

# %%
# We draw this time series by specifying the wanted marginal index (only 0 is available here).
graph0 = ts.drawMarginal(0)
graph0.setTitle("One ARMA realization")
graph0.setXTitle("t")
graph0.setYTitle(r"$X_t$")
view = otv.View(graph0)


# %%
# Generate a k time series
# k = 5
# myProcessSample = process.getSample(k)

# Then get the current state of the ARMA
# armaState = process.getState()
# print("armaState = ")
# print(armaState)

# %%
# We draw a sample of size 6 : it is six different time series.
size = 6
sample = process.getSample(size)
graph = sample.drawMarginal(0)
graph.setTitle("Six realizations of the ARMA process")
graph.setXTitle("t")
graph.setYTitle(r"$X_t$")
view = otv.View(graph)

# We can obtain the current state of the ARMA process :
armaState = process.getState()
print("ARMA state : ")
print(armaState)

# %%
# Note that if we use the process in the meantime and ask for the current state again, it will be different.
ts = process.getRealization()
armaState = process.getState()
print("ARMA state : ")
print(armaState)


# From the aforementioned `armaState`, we can get the last values of :math:`X_t` and the last values
# of the white noise :math:`E_t`.
myLastValues = armaState.getX()
print(myLastValues)
myLastEpsilonValues = armaState.getEpsilon()
print(myLastEpsilonValues)

# %%
# We have access to the number of iterations before getting a stationary state with
Ntherm = process.getNThermalization()
print("ThermalValue : %d" % Ntherm)

# %%
# This may be important to evaluate it with another precision epsilon :
epsilon = 1e-8
newThermalValue = process.computeNThermalization(epsilon)
print("newThermalValue : %d" % newThermalValue)
process.setNThermalization(newThermalValue)

# %%
# An important feature of an ARMA process is the future prediction from its current state over the next  `Nit` instants, say `Nit=20`.
Nit = 21

# %%
# First we specify a current state `armaState` and build the corresponding ARMA process `arma` :
arma = ot.ARMA(myARCoef, myMACoef, myWhiteNoise, armaState)

# Then, we generate a possible future. The last instant was :math:`t=0.9` so the future starts at
# :math:`t=1.0`. We represent the ARMA process with a solid line and its possible future as a dashed
# curve.
future = arma.getFuture(Nit)
graph = future.drawMarginal(0)
curve = graph.getDrawable(0)
curve.setLineStyle("dashed")
graph0.add(curve)
graph0.setTitle("One ARMA realization and a possible future")
# sphinx_gallery_thumbnail_number = 3
view = otv.View(graph0)


# %%
# It is of course possible to generate `N` different possible futures over the `Nit` next instants.
N = 6
possibleFuture_N = arma.getFuture(Nit, N)
possibleFuture_N.setName("Possible futures")

# %%
# Here we only show the future.
graph = possibleFuture_N.drawMarginal(0)
graph.setTitle("Six possible futures of the ARMA process")
graph.setXTitle("t")
graph.setYTitle(r"$X_t$")
view = otv.View(graph)

# %%
# Display all figures
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


# We create a numerical math function
myFunction = ot.SymbolicFunction(["E", "F", "L", "I"], ["-F*L^3/(3*E*I)"])

dim = myFunction.getInputDimension()
# We create a normal distribution point of dimension 1
mean = [50.0, 1.0, 10.0, 5.0]
sigma = ot.Point(dim, 1.0)
R = ot.IdentityMatrix(dim)
myDistribution = ot.Normal(mean, sigma, R)

# We create a 'usual' RandomVector from the Distribution
vect = ot.RandomVector(myDistribution)

# We create a composite random vector
output = ot.CompositeRandomVector(myFunction, vect)

# We create an Event from this RandomVector
myEvent = ot.ThresholdEvent(output, ot.Less(), -3.0)

# We create an AnalyticalResult based on fictive results
result = ot.AnalyticalResult(sigma, myEvent, False)

print("result=", result)
print("standard space design point=", result.getStandardSpaceDesignPoint())
print("physical space design point=", result.getPhysicalSpaceDesignPoint())
print(
    "is standard point origin in failure space? ",
    result.getIsStandardPointOriginInFailureSpace(),
)
print("importance factors=", result.getImportanceFactors())
print(
    "importance factors(classical)=",
    result.getImportanceFactors(ot.AnalyticalResult.CLASSICAL),
)
print(
    "importance factors(physical) =",
    result.getImportanceFactors(ot.AnalyticalResult.PHYSICAL),
)
print("Hasofer reliability index=%.5f" % result.getHasoferReliabilityIndex())
print("graph importance factors=", result.drawImportanceFactors())
print("graph sensitivity=", result.drawHasoferReliabilityIndexSensitivity())



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()

for xi in [-0.2, 0.0, 0.2]:
    # Instantiate one distribution object
    distribution = ot.GeneralizedPareto(1.5, xi, 0.5)

    print("Distribution ", repr(distribution))
    print("Distribution ", distribution)

    if xi > 0.0:
        print("asPareto=", distribution.asPareto())

    # Is this distribution elliptical ?
    print("Elliptical = ", distribution.isElliptical())

    # Is this distribution continuous ?
    print("Continuous = ", distribution.isContinuous())

    # Test for realization of distribution
    oneRealization = distribution.getRealization()
    print("oneRealization=", repr(oneRealization))

    # Test for sampling
    size = 10000
    oneSample = distribution.getSample(size)
    print("oneSample first=", repr(oneSample[0]), " last=", repr(oneSample[size - 1]))
    print("mean=", repr(oneSample.computeMean()))
    print("covariance=", repr(oneSample.computeCovariance()))

    # Define a point
    point = ot.Point(distribution.getDimension(), 1.5)
    print("Point= ", repr(point))

    # DDF = distribution.computeDDF( point )
    # print "ddf     =", repr(DDF)

    PDF = distribution.computePDF(point)
    print("pdf     = %.12g" % PDF)

    CDF = distribution.computeCDF(point)
    print("cdf= %.12g" % CDF)
    # CF = distribution.computeCharacteristicFunction( point[0] )
    # print "characteristic function=", CF
    PDFgr = distribution.computePDFGradient(point)
    print("pdf gradient     =", repr(PDFgr))

    CDFgr = distribution.computeCDFGradient(point)
    print("cdf gradient     =", repr(CDFgr))

    quantile = distribution.computeQuantile(0.95)
    print("quantile=", repr(quantile))
    print("cdf(quantile)=", distribution.computeCDF(quantile))
    # Get 95% survival function
    inverseSurvival = ot.Point(distribution.computeInverseSurvivalFunction(0.95))
    print("InverseSurvival=", repr(inverseSurvival))
    print(
        "Survival(inverseSurvival)=%.6f"
        % distribution.computeSurvivalFunction(inverseSurvival)
    )
    print("entropy=%.6f" % distribution.computeEntropy())

    # Confidence regions
    (
        interval,
        threshold,
    ) = distribution.computeMinimumVolumeIntervalWithMarginalProbability(0.95)
    print("Minimum volume interval=", interval)
    print("threshold=", ot.Point(1, threshold))
    levelSet, beta = distribution.computeMinimumVolumeLevelSetWithThreshold(0.95)
    print("Minimum volume level set=", levelSet)
    print("beta=", ot.Point(1, beta))
    (
        interval,
        beta,
    ) = distribution.computeBilateralConfidenceIntervalWithMarginalProbability(0.95)
    print("Bilateral confidence interval=", interval)
    print("beta=", ot.Point(1, beta))
    (
        interval,
        beta,
    ) = distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(
        0.95, False
    )
    print("Unilateral confidence interval (lower tail)=", interval)
    print("beta=", ot.Point(1, beta))
    (
        interval,
        beta,
    ) = distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(
        0.95, True
    )
    print("Unilateral confidence interval (upper tail)=", interval)
    print("beta=", ot.Point(1, beta))

    mean = distribution.getMean()
    print("mean=", repr(mean))
    standardDeviation = distribution.getStandardDeviation()
    print("standard deviation=", repr(standardDeviation))
    skewness = distribution.getSkewness()
    print("skewness=", repr(skewness))
    kurtosis = distribution.getKurtosis()
    print("kurtosis=", repr(kurtosis))
    covariance = distribution.getCovariance()
    print("covariance=", repr(covariance))
    parameters = distribution.getParametersCollection()
    print("parameters=", repr(parameters))
    print("Standard representative=", distribution.getStandardRepresentative())

    ot.Log.Show(ot.Log.TRACE)
    ot.RandomGenerator.SetSeed(1)
    validation = ott.DistributionValidation(distribution)
    validation.run()



"""
Visualize orthonormal polynomials basis
=======================================
"""

# %%
# In this example we are going to create some graphs useful after the launch of a polynomial chaos algorithm.
# More precisely, we draw some members of the 1-d polynomial family.

# %%
import openturns as ot
import openturns.viewer as otv


def drawFamily(factory, degreeMax=5):

    # Create a fine title
    titleJacobi = factory.__class__.__name__.replace("Factory", "") + " polynomials"

    # Create an empty graph which will be fulfilled
    # with curves
    graphJacobi = ot.Graph(titleJacobi, "z", "polynomial values")
    graphJacobi.setLegendPosition("upper right")

    # Fix the number of points for the graph
    pointNumber = 101

    # Bounds of the graph
    xMinJacobi = -1.0
    xMaxJacobi = 1.0

    # Get the curves
    for i in range(degreeMax):
        graphJacobi_temp = factory.build(i).draw(xMinJacobi, xMaxJacobi, pointNumber)
        graphJacobi_temp_draw = graphJacobi_temp.getDrawable(0)
        graphJacobi_temp_draw.setLegend("degree " + str(i))
        graphJacobi.add(graphJacobi_temp_draw)
    return graphJacobi


# %%
# Draw the 5-th first members of the Jacobi family.

# %%
# Instantiate a polynomial from the Jacobi family using the default Jacobi.ANALYSIS parameter set.
alpha = 0.5
beta = 1.5
jacobiFamily = ot.JacobiFactory(alpha, beta)
graph = drawFamily(jacobiFamily)
view = otv.View(graph)

# %%
laguerreFamily = ot.LaguerreFactory(2.75, 1)
graph = drawFamily(laguerreFamily)
view = otv.View(graph)

# %%
graph = drawFamily(ot.HermiteFactory())
view = otv.View(graph)

# %%
view.ShowAll()



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

b = ot.Basis()
seq = ot.BasisSequence(b)
print("sequence =", seq)
seq.add(ot.Indices())
b1 = seq.getBasis(0)
print(f"{b1=}")
dim = seq.getDimension()
print(f"{dim=}")
indices = seq.getIndices(0)
print(f"{indices=}")
masterB = seq.getMasterBasis()
print(f"{masterB=}")
size = seq.getSize()
print(f"{size=}")



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


distribution = ot.Triangular(-1.0, 0.3, 1.0)
algo = ot.OrthonormalizationAlgorithm(distribution)
print("algo=", algo)
print("measure=", repr(algo.getMeasure()))
algo.setMeasure(ot.Triangular(-1.0, -0.2, 1.0))
print("new measure=", repr(algo.getMeasure()))



#! /usr/bin/env python

import openturns as ot
from openturns.testing import assert_almost_equal

ot.TESTPREAMBLE()
ot.PlatformInfo.SetNumericalPrecision(5)

# We create a numerical math function
myFunction = ot.SymbolicFunction(["E", "F", "L", "I"], ["-F*L^3/(3*E*I)"])

dim = myFunction.getInputDimension()

# We create a normal distribution point of dimension 1
mean = [50.0, 1.0, 10.0, 5.0]
sigma = [1.0] * dim
R = ot.IdentityMatrix(dim)
myDistribution = ot.Normal(mean, sigma, R)

# We create a 'usual' RandomVector from the Distribution
vect = ot.RandomVector(myDistribution)

# We create a composite random vector
output = ot.CompositeRandomVector(myFunction, vect)

# We create an Event from this RandomVector
myEvent = ot.ThresholdEvent(output, ot.Less(), -3.0)

# We create a NearestPoint algorithm
myCobyla = ot.Cobyla()
myCobyla.setStartingPoint(mean)
myCobyla.setMaximumCallsNumber(400)
myCobyla.setMaximumAbsoluteError(1.0e-10)
myCobyla.setMaximumRelativeError(1.0e-10)
myCobyla.setMaximumResidualError(1.0e-10)
myCobyla.setMaximumConstraintError(1.0e-10)
print("myCobyla=", myCobyla)

# We create a FORM algorithm
# The first parameter is an OptimizationAlgorithm
# The second parameter is an event
# The third parameter is a starting point for the design point research
myAlgo = ot.FORM(myCobyla, myEvent)

print("FORM=", myAlgo)

# Check the starting point
point = myAlgo.getPhysicalStartingPoint()
myAlgo.setPhysicalStartingPoint(point)

# Perform the simulation
myAlgo.run()

# Stream out the iresult
result = myAlgo.getResult()
print("event probability=%.6f" % result.getEventProbability())
print("generalized reliability index=%.6f" % result.getGeneralisedReliabilityIndex())
print(
    "standard space design point=",
    result.getStandardSpaceDesignPoint(),
)
print(
    "physical space design point=",
    result.getPhysicalSpaceDesignPoint(),
)

# Is the standard point origin in failure space?
print(
    "is standard point origin in failure space? %s"
    % (result.getIsStandardPointOriginInFailureSpace() and "true" or "false")
)

print("importance factors=", result.getImportanceFactors())
print("Hasofer reliability index=%.6f" % result.getHasoferReliabilityIndex())

# Test with MultiStart algorithm
ot.RandomGenerator.SetSeed(0)

startingSample = myDistribution.getSample(10)

myMultiStartCobyla = ot.MultiStart(ot.Cobyla(), startingSample)
myMultiStartCobyla.setMaximumCallsNumber(400)
myMultiStartCobyla.setMaximumAbsoluteError(1.0e-5)
myMultiStartCobyla.setMaximumRelativeError(1.0e-5)
myMultiStartCobyla.setMaximumResidualError(1.0e-5)
myMultiStartCobyla.setMaximumConstraintError(1.0e-5)

# We create a FORM algorithm with MultiStart
algo_multistart = ot.FORM(myMultiStartCobyla, myEvent)

# Perform the simulation
algo_multistart.run()
result_multistart = algo_multistart.getResult()
assert_almost_equal(
    result_multistart.getEventProbability(), result.getEventProbability(), 1e-4, 0.0
)

# run twice
f = ot.SymbolicFunction(["x"], ["x-1.25"])
dist = ot.Normal(2.0, 0.5)
vect = ot.RandomVector(dist)
output = ot.CompositeRandomVector(f, vect)
event = ot.ThresholdEvent(output, ot.Less(), 0.0)
solver = ot.AbdoRackwitz()
solver.setStartingPoint(dist.getMean())
# -------------------------------------------------------
algo = ot.FORM(solver, event)
algo.run()
result = algo.getResult()
# -------------------------------------------------------
algo_2 = ot.FORM(solver, event)
algo_2.run()
result_2 = algo_2.getResult()
assert result.getEventProbability() == result_2.getEventProbability(), "wrong pf"



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott
import math as m

ot.TESTPREAMBLE()

# Instantiate one distribution object
for dim in range(1, 2):
    theta = ot.Point(dim + 1)
    for i in range(dim + 1):
        theta[i] = (i + 1.0) / 4.0
    distribution = ot.Dirichlet(theta)
    description = [""] * dim
    for j in range(1, dim + 1):
        oss = "Marginal " + str(j)
        description[j - 1] = oss
    distribution.setDescription(description)
    print("Distribution ", repr(distribution))
    print("Distribution ", distribution)

    # Is this distribution elliptical ?
    print("Elliptical = ", distribution.isElliptical())

    # Is this distribution continuous ?
    print("Continuous = ", distribution.isContinuous())

    # Test for realization of distribution
    oneRealization = distribution.getRealization()
    print("oneRealization=", repr(oneRealization))

    # Define a point
    point = ot.Point(distribution.getDimension(), 0.5 / distribution.getDimension())
    print("Point= ", repr(point))

    # Show PDF and CDF of point
    LPDF = distribution.computeLogPDF(point)
    print("log pdf= %.8g" % LPDF)
    PDF = distribution.computePDF(point)
    print("pdf     = %.8g" % PDF)
    CDF = distribution.computeCDF(point)
    print("cdf= %.8g" % CDF)
    quantile = distribution.computeQuantile(0.95)
    print("quantile=", repr(quantile))
    print("cdf(quantile)= %.6f" % distribution.computeCDF(quantile))
    # Get 95% survival function
    inverseSurvival = ot.Point(distribution.computeInverseSurvivalFunction(0.95))
    print("InverseSurvival=", repr(inverseSurvival))
    print(
        "Survival(inverseSurvival)=%.6f"
        % distribution.computeSurvivalFunction(inverseSurvival)
    )
    print("entropy=%.6f" % distribution.computeEntropy())

    # Confidence regions
    (
        interval,
        threshold,
    ) = distribution.computeMinimumVolumeIntervalWithMarginalProbability(0.95)
    print("Minimum volume interval=", interval)
    print("threshold=", ot.Point(1, threshold))
    levelSet, beta = distribution.computeMinimumVolumeLevelSetWithThreshold(0.95)
    print("Minimum volume level set=", levelSet)
    print("beta=", ot.Point(1, beta))
    (
        interval,
        beta,
    ) = distribution.computeBilateralConfidenceIntervalWithMarginalProbability(0.95)
    print("Bilateral confidence interval=", interval)
    print("beta=", ot.Point(1, beta))
    (
        interval,
        beta,
    ) = distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(
        0.95, False
    )
    print("Unilateral confidence interval (lower tail)=", interval)
    print("beta=", ot.Point(1, beta))
    (
        interval,
        beta,
    ) = distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(
        0.95, True
    )
    print("Unilateral confidence interval (upper tail)=", interval)
    print("beta=", ot.Point(1, beta))

    mean = distribution.getMean()
    print("mean=", repr(mean))
    standardDeviation = distribution.getStandardDeviation()
    print("standard deviation=", repr(standardDeviation))
    skewness = distribution.getSkewness()
    print("skewness=", repr(skewness))
    kurtosis = distribution.getKurtosis()
    print("kurtosis=", repr(kurtosis))
    covariance = distribution.getCovariance()
    print("covariance=", repr(covariance))

    x = 0.6
    y = [0.2] * (dim - 1)
    print("conditional PDF=%.6f" % distribution.computeConditionalPDF(x, y))
    print("conditional CDF=%.6f" % distribution.computeConditionalCDF(x, y))
    print("conditional quantile=%.6f" % distribution.computeConditionalQuantile(x, y))
    pt = ot.Point([0.1 * i + 0.15 for i in range(dim)])
    print(
        "sequential conditional PDF=",
        distribution.computeSequentialConditionalPDF(point),
    )
    resCDF = distribution.computeSequentialConditionalCDF(pt)
    print("sequential conditional CDF(", pt, ")=", resCDF)
    print(
        "sequential conditional quantile(",
        resCDF,
        ")=",
        distribution.computeSequentialConditionalQuantile(resCDF),
    )

    # Ticket #2306
    if dim == 2:
        condPDF = distribution.computeConditionalPDF(-0.1, [0.5])
        assert m.isfinite(condPDF), "condPDF is not normal"
        condPDF = distribution.computeConditionalPDF(0.5, [0.5])
        assert m.isfinite(condPDF), "condPDF is not normal"
        condPDF = distribution.computeConditionalPDF(0.6, [0.5])
        assert m.isfinite(condPDF), "condPDF is not normal"
        condPDF = distribution.computeSequentialConditionalPDF([0.5, -0.1])
        assert m.isfinite(condPDF), "condPDF is not normal"
        condPDF = distribution.computeSequentialConditionalPDF([0.5, 0.5])
        assert m.isfinite(condPDF), "condPDF is not normal"
        condPDF = distribution.computeSequentialConditionalPDF([0.5, 0.6])
        assert m.isfinite(condPDF), "condPDF is not normal"

    # Extract the marginals
    for i in range(dim):
        margin = distribution.getMarginal(i)
        print("margin=", margin)
        print("margin PDF= %.8g" % margin.computePDF(ot.Point(1, 0.5)))
        print("margin CDF= %.8g" % margin.computeCDF(ot.Point(1, 0.5)))
        print("margin quantile=", repr(margin.computeQuantile(0.95)))
        print("margin realization=", repr(margin.getRealization()))
    if dim >= 2:
        # Extract a 2-D marginal
        indices = [1, 0]
        print("indices=", indices)
        margins = distribution.getMarginal(indices)
        print("margins=", margins)
        print("margins PDF=", margins.computePDF(ot.Point(2, 0.5)))
        print("margins CDF= %.8g" % margins.computeCDF(ot.Point(2, 0.5)))
        quantile = margins.computeQuantile(0.95)
        print("margins quantile=", repr(quantile))
        print("margins CDF(quantile)= %.6f" % margins.computeCDF(quantile))
        print("margins realization=", repr(margins.getRealization()))

    ot.Log.Show(ot.Log.TRACE)
    validation = ott.DistributionValidation(distribution)
    validation.skipCDF()
    validation.skipGradient()
    validation.skipMoments()
    validation.run()



"""
Overview of univariate distribution management
==============================================
"""

# %%
# Abstract
# --------
#
# In this example, we present the following topics:
#
# - the distributions with several parametrizations (particularly with the `Beta` distribution),
# - the arithmetic of distributions and functions of distributions,
# - the :class:`~openturns.CompositeDistribution` for more general functions,
# - how to define our customized distribution with :class:`~openturns.PythonDistribution` if the distribution does not exist.

# %%
import openturns as ot
import openturns.viewer as otv


# %%
# Distributions with several parametrizations
# -------------------------------------------

# %%
# By default, any univariate distribution uses its native parameters. For some few distributions, alternative parameters might be used to define the distribution.
#
# For example, the :class:`~openturns.Beta` distribution has several parametrizations.
#
# The native parametrization uses the following parameters:
#
# - :math:`\alpha` : the first shape parameter, :math:`\alpha>0`,
# - :math:`\beta` : the second shape parameter, :math:`\beta> 0`,
# - :math:`a` : the lower bound,
# - :math:`b` : the upper bound with :math:`a<b`.
#
# The PDF of the Beta distribution is:
#
# .. math::
#    f(x) = \frac{(x-a)^{\alpha - 1} (b - x)^{\beta - 1}}{(b - a)^{\alpha + \beta - 1} B(\alpha, \beta)}
#
#
# for any :math:`x\in[a,b]`, where :math:`B` is Euler's Beta function.
#
# For any :math:`y,z>0`, the beta function is:
#
# .. math::
#    B(y,z) = \int_0^1 t^{y-1} (1-t)^{z-1} dt.
#

# %%
# The :class:`~openturns.Beta` class uses the native parametrization.

# %%
distribution = ot.Beta(2.5, 2.5, -1, 2)
graph = distribution.drawPDF()
view = otv.View(graph)

# %%
# The :class:`~openturns.BetaMuSigma` class provides another parametrization, based on the expectation :math:`\mu` and the standard deviation  :math:`\sigma` of the random variable:
#
# .. math::
#    \mu = a + \frac{(b-a)\alpha}{\alpha+\beta}
#
#
# and
#
# .. math::
#    \sigma^2 = \left(\frac{b-a}{\alpha+\beta}\right)^2 \frac{\alpha\beta}{\alpha+\beta+1}.
#
#
# Inverting the equations, we get:
#
#
# .. math::
#    \alpha =  \left(\dfrac{\mu-a}{b-a}\right) \left( \dfrac{(b-\mu)(\mu-a)}{\sigma^2}-1\right) \\
#
#
# and
#
# .. math::
#    \beta  =  \left( \dfrac{b-\mu}{\mu-a}\right) \alpha
#

# %%
# The following session creates a beta random variable with parameters :math:`\mu=0.2`, :math:`\sigma=0.6`, :math:`a=-1` et :math:`b=2`.

# %%
parameters = ot.BetaMuSigma(0.2, 0.6, -1, 2)
parameters.evaluate()

# %%
# The :class:`~openturns.ParametrizedDistribution` creates a distribution from a parametrization.

# %%
param_dist = ot.ParametrizedDistribution(parameters)
param_dist

# %%
# Functions of distributions
# --------------------------

# %%
# The library provides algebra of univariate distributions:
#
#  - `+, -`
#  - `*, /`
#
# It also provides methods to get the full distributions of `f(x)` where `f` can be equal to :
#
#  - `sin`,
#  - `cos`,
#  - `acos`,
#  - `asin`
#  - `square`,
#  - `inverse`,
#  - `sqrt`.

# %%
# In the following example, we create a beta and an exponential variable. Then we create the random variable equal to the sum of the two previous variables.

# %%
B = ot.Beta(5, 2, 9, 10)

# %%
E = ot.Exponential(3)

# %%
S = B + E

# %%
# The `S` variable is equipped with the methods of any distribution: we can for example compute the PDF or the CDF at any point and compute its quantile.
# For example, we can simply draw the PDF with the `drawPDF` method.

# %%
graph = S.drawPDF()
graph.setTitle("Sum of a beta and an exponential distribution")
view = otv.View(graph)

# %%
# The exponential function of this distribution can be computed with the `exp` method.

# %%
sumexp = S.exp()

# %%
graph = sumexp.drawPDF()
graph.setTitle("Exponential of a sum of a beta and an exponential")
view = otv.View(graph)

# %%
# The `CompositeDistribution` class for more general functions
# ------------------------------------------------------------
#
# More complex functions can be created thanks to the :class:`~openturns.CompositeDistribution` class, but it requires an `f` function.
# In the following example, we create the distribution of a random variable equal to the exponential of a Gaussian variable.
# Obviously, this is equivalent to the :class:`~openturns.LogNormal` distribution but this shows how such a distribution could be created.

# %%
# First, we create a distribution.

# %%
N = ot.Normal(0.0, 1.0)
N.setDescription(["Normal"])

# %%
# Secondly, we create a function.

# %%
f = ot.SymbolicFunction(["x"], ["exp(x)"])
f.setDescription(["X", "Exp(X)"])

# %%
# Finally, we create the distribution equal to the exponential of the Gaussian random variable.

# %%
dist = ot.CompositeDistribution(f, N)

# %%
graph = dist.drawPDF()
graph.setTitle("Exponential of a gaussian random variable")
view = otv.View(graph)

# %%
# In order to check the previous distribution, we compare it with the LogNormal distribution.

# %%
LN = ot.LogNormal()
LN.setDescription(["LogNormal"])
graph = LN.drawPDF()
view = otv.View(graph)


# %%
# The `PythonDistribution` class
# ------------------------------
#
# Another possibility is to define our own `distribution`.
#
# For example let us implement the `Quartic` kernel (also known as the `Biweight` kernel,
# see `here <https://en.wikipedia.org/wiki/Kernel_(statistics)#Kernel_functions_in_common_use>`_),
# which is sometimes used in the context of kernel smoothing.
# The PDF of the kernel is defined by:
#
# .. math::
#    f(u) = \frac{15}{16} (1 - u^2)^2
#
#
# for any :math:`u\in[-1,1]` and :math:`f(u)=0` otherwise.
#
# Expanding the previous square, we find:
#
# .. math::
#    f(u) = \frac{15}{16} (1 - 2 u^2 + u^4)
#
#
# for any :math:`u\in[-1,1]`.
#
# Integrating the previous equation leads to the CDF:
#
# .. math::
#    F(u) = \frac{1}{2} + \frac{15}{16} u - \frac{5}{8} u^3 + \frac{3}{16} u^5
#
#
# for any :math:`u\in[-1,1]` and :math:`F(u)=0` otherwise.
#
# The only required method is `computeCDF`. Since the PDF is easy to define in our example, we implement it as well.
# Here, the distribution is defined on the interval :math:`[-1,1]`, so that we define the `getRange` method.


# %%
class Quartic(ot.PythonDistribution):
    """
    Quartic (biweight) kernel
    See https://en.wikipedia.org/wiki/Kernel_(statistics)#Kernel_functions_in_common_use for more details
    """

    def __init__(self):
        super().__init__(1)
        self.c = 15.0 / 16

    def computeCDF(self, x):
        u = x[0]
        if u <= -1:
            p = 0.0
        elif u >= 1:
            p = 1.0
        else:
            p = 0.5 + 15.0 / 16 * u - 5.0 / 8 * pow(u, 3) + 3.0 / 16 * pow(u, 5)
        return p

    def computePDF(self, x):
        u = x[0]
        if u < -1 or u > 1:
            y = 0.0
        else:
            y = self.c * (1 - u**2) ** 2
        return y

    def getRange(self):
        return ot.Interval(-1, 1)


# %%
Q = ot.Distribution(Quartic())
Q.setDescription(["Quartic Kernel"])

# %%
graph = Q.drawPDF()
view = otv.View(graph)

# %%
# Display all graphs
otv.View.ShowAll()



"""
Bayesian calibration of the flooding model
==========================================
"""

# %%
#
# Abstract
# --------
#
# The goal of this example is to present the Bayesian calibration of the
# :ref:`flooding model<use-case-flood-model>`.
# We use the :class:`~openturns.RandomWalkMetropolisHastings` and
# :class:`~openturns.Gibbs` classes
# and simulate a sample of the posterior distribution using
# :ref:`metropolis_hastings`.

# %%
# Parameters to calibrate
# -----------------------
#
# The vector of parameters to calibrate is:
#
# .. math::
#    \vect{\theta} = (K_s,Z_v,Z_m).
#
#
# The reference values are:
#
# .. math::
#    K_s = 30, \qquad Z_v = 50, \qquad Z_m = 55.
#
#
# Observations
# ------------
#
# We consider the probabilistic model:
#
# .. math::
#    H =  \model(Q,K_s,Z_v,Z_m) + \epsilon
#
# where:
#
# .. math::
#    \epsilon \sim \mathcal{N}(0,\sigma^2)
#
# Thus, the error of the water height follows a normal distribution with a zero mean and a standard variation :math:`\sigma`.
#
# We have some observations :math:`\{(Q_i,h_i)\}_{i=1,...,\sampleSize}`. Each observation is a couple made of the flowrate
# and the corresponding river height. We assume that the observations are associated to independent errors.
# We consider a sample size equal to:
#
# .. math::
#    \sampleSize = 10.
#
#
# Variables
# ---------
#
# - :math:`Q` : Input. Observed.
# - :math:`K_s`, :math:`Z_v`, :math:`Z_m` : Input. Calibrated.
# - :math:`h`: Output. Observed.
#
# Analysis
# --------
#
# In the description of the :ref:`flooding model<use-case-flood-model>`, we see that only one parameter
# can be identified.
# Hence, calibrating this model requires some regularization.
# In this example, we use Bayesian methods as a way to regularize the model.

# %%
# Generate the observations
# -------------------------

# %%
from matplotlib import pyplot as plt
from openturns.usecases import flood_model
import openturns.viewer as otv
import numpy as np
import openturns as ot

# %%
# A basic implementation of the probabilistic model is available in the usecases module :
fm = flood_model.FloodModel()

# %%
# We define the model :math:`\model` which has 4 inputs and one output :math:`h`.
#
# The non linear least squares algorithm does not take into account bounds in the parameters.
# Therefore, we ensure that the output is computed whatever the inputs.
# The model fails into two situations:
#
# * if :math:`K_s<0`,
# * if :math:`Z_v-Z_m<0`.
#
# In these cases, we return an infinite number.

# %%


def functionFlooding(X):
    L = 5.0e3
    B = 300.0
    Q, K_s, Z_v, Z_m = X
    alpha = (Z_m - Z_v) / L
    if alpha < 0.0 or K_s <= 0.0:
        H = np.inf
    else:
        H = (Q / (K_s * B * np.sqrt(alpha))) ** (3.0 / 5.0)
    return [H]


# %%
g = ot.PythonFunction(4, 1, functionFlooding)
g = ot.MemoizeFunction(g)
g.setOutputDescription(["H (m)"])

# %%
# We load the input distribution for the flow :math:`Q`.
Q = fm.Q

# %%
# Set the parameters to be calibrated.

# %%
K_s = ot.Dirac(30.0)
Z_v = ot.Dirac(50.0)
Z_m = ot.Dirac(55.0)
K_s.setDescription(["Ks (m^(1/3)/s)"])
Z_v.setDescription(["Zv (m)"])
Z_m.setDescription(["Zm (m)"])

# %%
# We create the joint input distribution of :math:`(Q, K_s, Z_v, Z_m)`.

# %%
inputRandomVector = ot.JointDistribution([Q, K_s, Z_v, Z_m])

# %%
# We import some noisy observations of the flow rate (column 0) and the height (column 1) already
# stored of the  :ref:`flooding model<use-case-flood-model>` in the field *data*.
Q_H_obs = fm.data
Qobs = Q_H_obs.getMarginal(0)
Hobs = Q_H_obs.getMarginal(1)

# %%
graph = ot.Graph("Observations", "Q (m3/s)", "h (m)")
cloud = ot.Cloud(Q_H_obs)
# cloud = ot.Cloud(Qobs, Hobs)
graph.add(cloud)
view = otv.View(graph)


# %%
# Setting the calibration parameters
# ----------------------------------

# %%
# We assume that the output of the model :math:`H` is random and follows a normal distribution with the standard
# deviation :math:`\sigma = 0.5` and centered on :math:`\mu = \model(Q, K_s, Z_v, Z_m)`.
#
# Then, we define the parametric model :math:`\vect z = f_Q(\vect\theta)` that associates each observation
# :math:`Q` and value of the parameters :math:`\vect \theta = (K_s, Z_v, Z_m)`
# to the parameters :math:`\vect z=(\mu, \sigma)`.
# We want to get the posterior distribution of the random vector :math:`\vect \Theta` that fits the best to the
# observations :math:`(Q_i, h_i)_i`.


# %%
def fullModelPy(X):
    Q, K_s, Z_v, Z_m = X
    mu = g(X)[0]
    sigma = 0.5  # (m^2) The standard deviation of the observation error.
    return [mu, sigma]


fullModel = ot.PythonFunction(4, 2, fullModelPy)
linkFunction = ot.ParametricFunction(fullModel, [0], [np.nan])
print(linkFunction)

# %%
# We define the prior distribution of  :math:`\vect\Theta`. We assume that it is a normal distribution.
# In the Bayesian framework, this is called the *prior* normal distribution. In the data assimilation framework, this
# is called the *background*.
# We assume that the prior distribution has independent components, that the mean marginal values are equal to the reference values
# and that the marginal standard deviations are known.

# %%
KsInitial = 20.0
ZvInitial = 49.0
ZmInitial = 51.0
parameterPriorMean = [KsInitial, ZvInitial, ZmInitial]
paramDim = len(parameterPriorMean)

# %%
sigmaKs = 5.0
sigmaZv = 1.0
sigmaZm = 1.0

# %%
parameterPriorCovariance = ot.CovarianceMatrix(paramDim)
parameterPriorCovariance[0, 0] = sigmaKs**2
parameterPriorCovariance[1, 1] = sigmaZv**2
parameterPriorCovariance[2, 2] = sigmaZm**2

# %%
# Define the prior distribution of :math:`\vect\Theta`.
prior = ot.Normal(parameterPriorMean, parameterPriorCovariance)
prior.setDescription(["Ks", "Zv", "Zm"])

# %%
# Define the distribution of the output :math:`H | \vect{z}` conditional on model predictions.
# Note that its parameter dimension is the one of :math:`\vect{z}`, so the model must be adjusted accordingly.
# In other words, the input argument of the `setParameter` method of the conditional distribution must be equal to the dimension of the output of the `model`.
# Hence, we do not have to set the actual parameters: only the type of distribution is used.

# %%
conditional = ot.Normal()

# %%
# The proposed steps for :math:`K_s` :math:`Z_v` and :math:`Z_m`
# will all follow uniform distributions,
# but with different supports.

# %%
proposal = [ot.Uniform(-5.0, 5.0), ot.Uniform(-1.0, 1.0), ot.Uniform(-1.0, 1.0)]

# %%
# Build a Gibbs sampler
# ---------------------
initialState = parameterPriorMean
mh_coll = [
    ot.RandomWalkMetropolisHastings(prior, initialState, proposal[i], [i])
    for i in range(paramDim)
]
for mh in mh_coll:
    mh.setLikelihood(conditional, Hobs, linkFunction, Qobs)
sampler = ot.Gibbs(mh_coll)

# %%
# Generate a sample from the posterior distribution of :math:`\vect \Theta`.
sampleSize = 1000
sample = sampler.getSample(sampleSize)

# %%
# Look at the acceptance rates of the random walk Metropolis-Hastings samplers.
[mh.getAcceptanceRate() for mh in sampler.getMetropolisHastingsCollection()]

# %%
# Build the posterior distribution of :math:`\vect \Theta` by kernel smoothing.
kernel = ot.KernelSmoothing()
posterior = kernel.build(sample)

# %%
# Display prior vs posterior for each parameter.


def plot_bayesian_prior_vs_posterior_pdf(prior, posterior):
    """
    Plot the prior and posterior distribution of a Bayesian calibration

    Parameters
    ----------
    prior : ot.Distribution(dimension)
        The prior.
    posterior : ot.Distribution(dimension)
        The posterior.

    Return
    ------
    grid : ot.GridLayout(1, dimension)
        The prior and posterior PDF for each marginal.
    """
    paramDim = prior.getDimension()
    grid = ot.GridLayout(1, paramDim)
    parameterNames = prior.getDescription()
    for parameter_index in range(paramDim):
        graph = ot.Graph("", parameterNames[parameter_index], "PDF")
        # Prior
        curve = prior.getMarginal(parameter_index).drawPDF().getDrawable(0)
        curve.setLineStyle(
            ot.ResourceMap.GetAsString("CalibrationResult-PriorLineStyle")
        )
        curve.setLegend("Prior")
        graph.add(curve)
        # Posterior
        curve = posterior.getMarginal(parameter_index).drawPDF().getDrawable(0)
        curve.setLineStyle(
            ot.ResourceMap.GetAsString("CalibrationResult-PosteriorLineStyle")
        )
        curve.setLegend("Posterior")
        graph.add(curve)
        #
        if parameter_index < paramDim - 1:
            graph.setLegends([""])
        if parameter_index > 0:
            graph.setYTitle("")
        graph.setLegendPosition("upper right")
        grid.setGraph(0, parameter_index, graph)
    grid.setTitle("Bayesian calibration")
    return grid


# %%
# sphinx_gallery_thumbnail_number = 2
grid = plot_bayesian_prior_vs_posterior_pdf(prior, posterior)
otv.View(
    grid,
    figure_kw={"figsize": (8.0, 3.0)},
    legend_kw={"bbox_to_anchor": (1.0, 1.0), "loc": "upper left"},
)
plt.subplots_adjust(right=0.8, bottom=0.2, wspace=0.3)

# %%
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot
import numpy as np
import os

ot.TESTPREAMBLE()


sample = ot.Normal(3).getSample(1000)
tree = ot.NaiveNearestNeighbour(sample)
print("tree=", tree)

test = ot.Normal(3).getSample(100)

sample_np = np.array(sample)
test_np = np.array(test)


def nearest_debug(x):
    a = np.sum(np.square(sample_np - x), axis=1)
    return int(np.argmin(a))


def nearest_debug_indices(x):
    a = np.sum(np.square(sample_np - x), axis=1)
    return a.argsort()


neighbourIndices = tree.query(test)

neighbourIndices_np = [nearest_debug(x) for x in test]

if neighbourIndices != neighbourIndices_np:
    print("Errors found in query")
    os.exit(1)

for x in test:
    if np.any(nearest_debug_indices(x)[:10] != tree.queryK(x, 10, True)):
        print("Errors found in queryK")
        os.exit(1)



"""
Create a compound random vector
===============================
"""

# %%
# In this example we are going to build the random vector :math:`\inputRV` defined as the
# compound distribution of:
#
# .. math::
#
#    \inputRV|\vect{\Theta}
#
# with respect to parameter random vector :math:`\vect{\Theta}` following the distribution :math:`\cL_{\vect{\Theta}}`.
#
# This example creates a :class:`~openturns.CompoundRandomVector`. Remember that a :class:`~openturns.CompoundRandomVector`
# (and more generally a :class:`~openturns.RandomVector`) can only be sampled.
#
# There is no restriction on the random vector :math:`\vect{\Theta}`. It can be created from any multivariate distribution or
# as the output of a function :math:`f` applied to an input random vector :math:`\vect{Y}`: :math:`\vect{\Theta} = f(\vect{Y})`.
#
# Note that in some restricted cases, it is possible to create the
# distribution of :math:`\inputRV` using the class :class:`~openturns.CompoundDistribution`.
# The :class:`~openturns.CompoundDistribution` offers a lot of methods attached to the distribution, in particular the
# computation of the PDF, CDF, the moments if any, :math:`\dots`. The advantage of the :class:`~openturns.CompoundRandomVector` relies
# on the fact that it is fast to build and can be built in all cases. But it only offers the sampling capacity.
#
# We consider the following cases:
#
# - Case 1: the parameter random vector has continuous marginals,
# - Case 2: the parameter random vector has dependent continuous and discrete marginals,
# - Case 3: the parameter random vector has any dependent marginals.

# %%
import openturns as ot
import openturns.viewer as otv

# %%
# Case 1: the parameter random vector  has continuous marginals
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
# We consider the case where :math:`X` is of dimension 1 and follows an exponential distribution
# defined by:
#
# ================================ =========================================================  ====================================
# Variable                         Distribution                                               Parameter
# ================================ =========================================================  ====================================
# :math:`X`                        :class:`~openturns.Exponential` (:math:`\Lambda, \Gamma`)  (:math:`\Lambda, \Gamma`) are random
# :math:`\Lambda`                  :class:`~openturns.Uniform` (:math:`a, b`)                 :math:`(a,b) = (0, 1)`
# :math:`\Gamma`                   :class:`~openturns.Uniform` (:math:`a, b`)                 :math:`(a,b) = (1, 2)`
# Copula (:math:`\Lambda, \Gamma`) :class:`~openturns.ClaytonCopula` (:math:`\theta`)         :math:`\theta = 2`
# ================================ =========================================================  ====================================
#

# %%
# Create the parameter random vector :math:`\vect{\Theta} = (\Lambda, \Gamma)`:
lambdaDist = ot.Uniform(0.0, 1.0)
gammaDist = ot.Uniform(1.0, 2.0)
thetaDist = ot.JointDistribution([lambdaDist, gammaDist], ot.ClaytonCopula(2))
thetaRV = ot.RandomVector(thetaDist)

# %%
# Create the :math:`\inputRV|\vect{\Theta}` distribution: as the parameters have no importance, we
# create the default distribution.
XgivenThetaDist = ot.Exponential()

# %%
# Create the :math:`\inputRV` random vector.
X_RV = ot.CompoundRandomVector(XgivenThetaDist, thetaRV)

# %%
# Draw a sample
X_RV.getSample(5)

# %%
# If we generate a large sample of the random vector, we can fit its distribution with non-parametric techniques
# such as a kernel smoothing.
sample_large = X_RV.getSample(10000)
dist_KS = ot.KernelSmoothing().build(sample_large)
g_PDF = dist_KS.drawPDF()
g_PDF.setTitle("Case 1: PDF of X by kernel smoothing")
g_PDF.setXTitle("x")
g_PDF.setLegendPosition("")
view = otv.View(g_PDF)

# %%
# Case 2: the parameter random vector has dependent continuous and discrete marginals
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
# We consider the case where :math:`X` is of dimension 1 and follows an exponential distribution
# defined by:
#
# =================================  =========================================================  ====================================
# Variable                           Distribution                                               Parameter
# =================================  =========================================================  ====================================
# :math:`X`                          :class:`~openturns.Exponential` (:math:`\Lambda, \Gamma`)  (:math:`\Lambda, \Gamma`) are random
# :math:`\Lambda`                    1 + :class:`~openturns.Poisson` (:math:`\ell`)             :math:`\ell = 1`
# :math:`\Gamma`                     :class:`~openturns.Uniform` (:math:`a, b`)                 :math:`(a,b) = (1, 2)`
# Copula  (:math:`\Lambda, \Gamma`)  :class:`~openturns.ClaytonCopula` (:math:`\theta`)         :math:`\theta = 2`
# =================================  =========================================================  ====================================
#

# %%
# Create the parameter random vector :math:`\vect{\Theta} = (\Lambda, \Gamma)`. We shift the Poisson distribution to
# get positive values for :math:`\Lambda`.
lambdaDist = 1 + ot.Poisson(1)
gammaDist = ot.Uniform(1.0, 2.0)
thetaDist = ot.JointDistribution([lambdaDist, gammaDist], ot.ClaytonCopula(2))
thetaRV = ot.RandomVector(thetaDist)

# %%
# Create the :math:`\inputRV|\vect{\Theta}` random vector.
XgivenThetaDist = ot.Exponential()
X_RV = ot.CompoundRandomVector(XgivenThetaDist, thetaRV)

# %%
# If we generate a large sample of the random vector, we can fit its distribution with non-parametric techniques
# such as a kernel smoothing.
sample_large = X_RV.getSample(10000)
dist_KS = ot.KernelSmoothing().build(sample_large)
g_PDF = dist_KS.drawPDF()
g_PDF.setTitle("Case 2: PDF of X by kernel smoothing")
g_PDF.setXTitle("x")
g_PDF.setLegendPosition("")
view = otv.View(g_PDF)

# %%
# Case 3: the parameter random vector has any dependent marginals
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
# We consider the case where :math:`X` is of dimension 1 and follows an exponential distribution
# defined by:
#
# =================================  =========================================================  ====================================
# Variable         Distribution                                               Parameter
# =================================  =========================================================  ====================================
# :math:`X`                          :class:`~openturns.Exponential` (:math:`\Lambda, \Gamma`)  (:math:`\Lambda, \Gamma`) are random
# :math:`\Lambda`                    :class:`~openturns.Mixture`                                see below
# :math:`\Gamma`                     :class:`~openturns.Uniform` (:math:`a, b`)                 :math:`(a,b) = (1, 2)`
# Copula  (:math:`\Lambda, \Gamma`)  :class:`~openturns.ClaytonCopula` (:math:`\theta`)         :math:`\theta = 2`
# =================================  =========================================================  ====================================
#
# where the mixture is built from the :class:`~openturns.Exponential` (:math:`\ell`) with :math:`\ell = 1` and the
# :class:`~openturns.Geometric` (:math:`p`) with :math:`p = 0.1`, with equal weights. In this case, the distribution of :math:`\Lambda` is
# not discrete nor continuous.

# %%
# Create the parameter random vector :math:`\vect{\Theta} = (\Lambda, \Gamma)`:
lambdaDist = ot.Mixture([ot.Exponential(1.0), ot.Geometric(0.1)])
gammaDist = ot.Uniform(1.0, 2.0)
thetaDist = ot.JointDistribution([lambdaDist, gammaDist], ot.ClaytonCopula(2))
thetaRV = ot.RandomVector(thetaDist)

# %%
# Create the :math:`\inputRV|\vect{\Theta}` random vector.
XgivenThetaDist = ot.Exponential()
X_RV = ot.CompoundRandomVector(XgivenThetaDist, thetaRV)

# %%
# If we generate a large sample of the random vector, we can fit its distribution with non-parametric techniques
# such as a kernel smoothing.
sample_large = X_RV.getSample(10000)
dist_KS = ot.KernelSmoothing().build(sample_large)
g_PDF = dist_KS.drawPDF()
g_PDF.setTitle("Case 3: PDF of X by kernel smoothing")
g_PDF.setXTitle("x")
g_PDF.setLegendPosition("")
view = otv.View(g_PDF)

# %%
view.ShowAll()



import openturns as ot
from matplotlib import pyplot as plt
import openturns.viewer as otv

# Create a process on a regular grid
myGrid = ot.RegularGrid(0.0, 0.1, 100)

amplitude = [5.0]
scale = [0.2]
myCovModel = ot.ExponentialModel(scale, amplitude)
myXProcess = ot.GaussianProcess(myCovModel, myGrid)

# Create a trend
fTrend = ot.SymbolicFunction(["t"], ["1+2*t+t^2"])
fTemp = ot.TrendTransform(fTrend, myGrid)

# Add the trend to the process and get a field
myYProcess = ot.CompositeProcess(fTemp, myXProcess)
myYField = myYProcess.getRealization()

# Create a TrendFactory
myBasisSequenceFactory = ot.LARS()
myFittingAlgorithm = ot.KFold()
func1 = ot.SymbolicFunction(["t"], ["1"])
func2 = ot.SymbolicFunction(["t"], ["t"])
func3 = ot.SymbolicFunction(["t"], ["t^2"])
myBasis = ot.Basis([func1, func2, func3])

myTrendFactory = ot.TrendFactory(myBasisSequenceFactory, myFittingAlgorithm)

# Estimate the trend
myTrendTransform = myTrendFactory.build(myYField, myBasis)


graph = myTrendTransform.getTrendFunction().draw(0.0, 10.0)
graph.add(fTrend.draw(0.0, 10.0))
graph.add(ot.Cloud(myYField.getMesh().getVertices(), myYField.getValues()))
graph.setColors(["red", "blue", "black"])
graph.setLegends(["estimated trend", "real trend", "sample"])
graph.setLegendPosition("upper left")
graph.setYTitle("values")
graph.setTitle("Trend estimation from a field")

fig = plt.figure(figsize=(8, 4))
axis = fig.add_subplot(111)
axis.set_xlim(auto=True)

otv.View(graph, figure=fig, axes=[axis], add_legend=True)



"""
Create a design of experiments with discrete and continuous variables
=====================================================================
"""

# %%
# In this example we present how to create a design of experiments when one (or several) of the marginals are discrete.

# %%
import openturns as ot
import openturns.viewer as otv

# %%
# To create the first marginal of the distribution, we select a univariate discrete distribution.
# Some of them, like the `Bernoulli` or `Geometric` distributions, are implemented in the library as classes.
# In this example however, we pick the `FiniteDiscreteDistribution` distribution that assigns equal weights to the values -2, -1, 1 and 2.

# %%
sample = ot.Sample([[-2.0], [-1.0], [1.0], [2.0]])
sample

# %%
X0 = ot.FiniteDiscreteDistribution(sample)

# %%
# For the second marginal, we pick a Gaussian distribution.

# %%
X1 = ot.Normal()

# %%
# Create the multivariate distribution from its marginals and an independent copula.

# %%
distribution = ot.JointDistribution([X0, X1])

# %%
# Create the design.

# %%
size = 100
experiment = ot.MonteCarloExperiment(distribution, size)
sample = experiment.generate()

# %%
# Plot the design.

# %%
graph = ot.Graph("MonteCarloExperiment", "x0", "x1")
cloud = ot.Cloud(sample, "blue", "fsquare", "")
graph.add(cloud)
view = otv.View(graph)

# %%
# Any other type of design of experiments can be generated based on this distribution. The following example shows a LHS experiment.

# %%
size = 100
alwaysShuffle = True
randomShift = True
experiment = ot.LHSExperiment(distribution, size, alwaysShuffle, randomShift)
sample = experiment.generate()

# %%
graph = ot.Graph("LHSExperiment", "x0", "x1")
cloud = ot.Cloud(sample, "blue", "fsquare", "")
graph.add(cloud)
view = otv.View(graph)

# %%
# Display all figures
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

# Gamma related functions
# pGamma
kMin = 0.2
kMax = 5.0
nK = 5
xMin = 0.1
xMax = 0.9
nX = 5
for i1 in range(nK):
    k = kMin + (kMax - kMin) * i1 / (nK - 1)
    for iX in range(nX):
        x = xMin + (xMax - xMin) * iX / (nX - 1)
        print(
            "pGamma(",
            k,
            ",  %.12g" % x,
            ")=%.6g" % ot.DistFunc.pGamma(k, x),
            ", complementary=%.6g" % ot.DistFunc.pGamma(k, x, True),
        )
# qGamma
kMin = 0.2
kMax = 5.0
nK = 5
qMin = 0.1
qMax = 0.9
nQ = 5
for i1 in range(nK):
    k = kMin + (kMax - kMin) * i1 / (nK - 1)
    for iQ in range(nQ):
        q = qMin + (qMax - qMin) * iQ / (nQ - 1)
        print(
            "qGamma(",
            k,
            ",  %.12g" % q,
            ")=%.6g" % ot.DistFunc.qGamma(k, q),
            ", complementary=%.6g" % ot.DistFunc.qGamma(k, q, True),
        )
# rGamma
kMin = 0.2
kMax = 5.0
nK = 5
nR = 5
for i1 in range(nK):
    k = kMin + (kMax - kMin) * i1 / (nK - 1)
    for iR in range(nR):
        print("rGamma(", k, ")=%.6g" % ot.DistFunc.rGamma(k))



"""
Getting started
===============
"""

# %%
# The goal of this example is to highlight the main features of the library.
# It assumes a basic knowledge of the few concepts of the uncertainty methodology (inference, probabilistic modelling, simulation, sensitivity).

# %%
from openturns.usecases import cantilever_beam
import openturns as ot
import openturns.viewer as otv


# %%
# Inference of the input distribution
# -----------------------------------

# %%
# Load a sample of the input variables from the cantilever beam use-case
cb = cantilever_beam.CantileverBeam()
data = cb.data
print(data[:5])

# %%
# Infer marginals from most common 1-d parametric distributions
marginal_factories = [
    ot.NormalFactory(),
    ot.BetaFactory(),
    ot.UniformFactory(),
    ot.LogNormalFactory(),
    ot.TriangularFactory(),
    ot.WeibullMinFactory(),
    ot.WeibullMaxFactory(),
]
estimated_marginals = []
for index in range(data.getDimension()):
    best_model, _ = ot.FittingTest.BestModelBIC(data[:, index], marginal_factories)
    print(best_model)
    estimated_marginals.append(best_model)
print(estimated_marginals)

# %%
# Assess the goodness of fit of the second marginal
graph = ot.VisualTest.DrawQQplot(data[:, 1], estimated_marginals[1])
_ = otv.View(graph)

# %%
# Infer the copula from common n-d parametric copulas in the ranks space
# If the copula is known it can be provided directly through :class:`~openturns.NormalCopula` for example
copula_factories = [
    ot.IndependentCopulaFactory(),
    ot.NormalCopulaFactory(),
    ot.StudentCopulaFactory(),
]
copula_sample = ot.Sample(data.getSize(), data.getDimension())
for index in range(data.getDimension()):
    copula_sample[:, index] = estimated_marginals[index].computeCDF(data[:, index])
estimated_copula, _ = ot.FittingTest.BestModelBIC(copula_sample, copula_factories)
print(estimated_copula)

# %%
# Assemble the joint distribution from marginals and copula
estimated_distribution = ot.JointDistribution(estimated_marginals, estimated_copula)
print(estimated_distribution)

# %%
# Uncertainty propagation
# -----------------------

# %%
# Creation of the model


def fpython(X):
    E, F, L, II = X
    Y = F * L**3 / (3 * E * II)
    return [Y]


model = ot.PythonFunction(4, 1, fpython)


# %%
# The distribution can also be given directly when known
distribution = cb.independentDistribution

# %%
# Propagate the input distribution through the model
# Here the function evaluation can benefit parallelization depending on its type, see also n_cpus option from :class:`~openturns.PythonFunction`.
sample_x = distribution.getSample(1000)
sample_y = model(sample_x)

# %%
# Estimate a non-parametric distribution (see :class:`~openturns.KernelSmoothing`) of the output variable
ks = ot.KernelSmoothing().build(sample_y)
grid = ot.GridLayout(1, 2)
grid.setGraph(0, 0, ks.drawPDF())
grid.setGraph(0, 1, ks.drawCDF())
_ = otv.View(grid)


# %%
# Build a metamodel with polynomial chaos expansion
# -------------------------------------------------
algo = ot.LeastSquaresExpansion(sample_x, sample_y, distribution)
algo.run()
metamodel_result = algo.getResult()
metamodel = metamodel_result.getMetaModel()
test_x = distribution.getSample(1000)
test_y = model(test_x)
predictions = metamodel(test_x)
validation = ot.MetaModelValidation(test_y, predictions)
graph = validation.drawValidation()
graph.setTitle(graph.getTitle() + f" R2={validation.computeR2Score()}")
_ = otv.View(graph)

# %%
# Sensitivity analysis
# ----------------------

# %%
# For simplicity we can use a method that does not impose special requirements on the design of experiments
sobol_x = distribution.getSample(5000)
sobol_y = metamodel(sobol_x)
algo = ot.RankSobolSensitivityAlgorithm(sobol_x, sobol_y)
print(algo.getFirstOrderIndices())

# %%
# Plot the sensitivity indices
# The most influential variable is `F`.
graph = algo.draw()
_ = otv.View(graph)

# %%
otv.View.ShowAll()



#! /usr/bin/env python

import os
import openturns as ot

ot.TESTPREAMBLE()


class FUNC(ot.OpenTURNSPythonFunction):
    def __init__(self):
        ot.OpenTURNSPythonFunction.__init__(self, 2, 1)
        self.setInputDescription(["R", "S"])
        self.setOutputDescription(["T"])

    def _exec(self, X):
        return [X[0] + X[1]]


F = FUNC()

# Instance creation
f1 = ot.Function(F)


def mul(X):
    return [X[0] * X[1] * X[2]]


f2 = ot.PythonFunction(3, 1, mul)


st = ot.Study()
fileName = "PyNMF.xml"
st.setStorageManager(ot.XMLStorageManager(fileName))

st.add("f1", f1)
st.add("f2", f2)

st.save()

print("saved f1=", f1)
print("saved f2=", f2)

f1 = ot.Function()
f2 = ot.Function()

st = ot.Study()
st.setStorageManager(ot.XMLStorageManager(fileName))

st.load()

st.fillObject("f1", f1)
st.fillObject("f2", f2)
print("loaded f1=", f1)
print("loaded f2=", f2)


inPt = ot.Point(2, 2.0)
outPt = f1(inPt)
print(repr(outPt))

outPt = f1((10.0, 11.0))
print(repr(outPt))

inSample = ot.Sample(10, 2)
for i in range(10):
    inSample[i] = ot.Point((i, i))
print(repr(inSample))

outSample = f1(inSample)
print(repr(outSample))

outSample = f1(((100.0, 100.0), (101.0, 101.0), (102.0, 102.0)))
print(repr(outSample))

os.remove(fileName)



"""
A quick start guide to contours
===============================
"""

# sphinx_gallery_thumbnail_number = 6
# %%
#
# In this example we show how to create contour graphs and how to make the most of their display settings.

# %%
# The `draw` method, the `Graph` and `Contour` classes
# ----------------------------------------------------
#
# The simplest way to create a contour graph is to use the `draw` method of a bidimensional function.

# %%
import openturns as ot
import openturns.viewer as otv

# %%
# We build a bidimensional function (function of `x` and `y`), define the study domain and the sample size

# %%
f = ot.SymbolicFunction(["x", "y"], ["exp(-sin(cos(y)^2 * x^2 + sin(x)^2 * y^2))"])
XMin = -5.0
XMax = 5.0
YMin = -5.0
YMax = 5.0
NX = 75
NY = 75

# %%
# We build the graph by calling the `draw` method and display it

# %%
graph = f.draw([XMin, YMin], [XMax, YMax], [NX, NY])
view = otv.View(graph)

# %%
# The graph contains a unique drawable whose implementation is of class :class:`~openturns.Contour`

# %%
contour = graph.getDrawable(0).getImplementation()
print(type(contour).__name__)

# %%
# Another way to build the contour is to build the data sample and give it to the constructor of the :class:`~openturns.Contour` class

# %%
inputData = ot.Box([NX, NY]).generate()
inputData *= [XMax - XMin, YMax - YMin]
inputData += [XMin, YMin]
data = f(inputData)

x = ot.Box([NX]).generate() * [XMax - XMin] + [XMin]
y = ot.Box([NY]).generate() * [YMax - YMin] + [YMin]
contour = ot.Contour(x, y, data)

# %%
# By creating an empty graph and adding the contour we can display the whole.

# %%
graph = ot.Graph("Complex iso lines", "u1", "u2")
graph.add(contour)
view = otv.View(graph)

# %%
# The previous graph does not show the associated color bar.
# This point can be modified.
# We will also change the color map, the number of contour lines and hide the labels.

# %%
contour.setColorBarPosition("right")
contour.setColorMap("inferno")
contour.buildDefaultLevels(5)
contour.setDrawLabels(False)
graph.setDrawables([ot.Drawable(contour)])
view = otv.View(graph)

# %%
# For such a function, contour lines are not easy to interpret.
# We will modify the contour to use filled areas.
contour.setIsFilled(True)
graph.setTitle("Complex filled contour")
graph.setDrawables([ot.Drawable(contour)])
view = otv.View(graph)

# %%
# Sometimes the colors are not very distinct because some levels are very close while others are very far apart.
# In this case, it is possible to add hatching to the surfaces.
# Here we will also use transparency to soften the colors.

# %%
contour.setAlpha(0.3)
contour.setHatches(["/", "\\", "/\\", "+", "*"])
graph.setTitle("Complex filled contour with hatches")
graph.setDrawables([ot.Drawable(contour)])
view = otv.View(graph)

# %%
# When the function takes values very different in magnitude, it may be useful to change the norm which is
# used to distribute the colors and to bound the color interval.
# Here we will also let `Matplotlib` calculate the levels by not giving any level to the contour

# %%
contour.setColorMapNorm("log")
contour.setLevels([])
contour.setExtend("neither")
contour.setVmin(0.5)
contour.setVmax(2)
graph.setTitle("Complex contour with log norm and automatic levels")
graph.setDrawables([ot.Drawable(contour)])
view = otv.View(graph)

# %%
# These capabilities can also be leveraged for distributions.
# We build here 2 distributions, Funky and Punk, which we mix.

# %%
corr = ot.CorrelationMatrix(2)
corr[0, 1] = 0.2
copula = ot.NormalCopula(corr)
x1 = ot.Normal(-1.0, 1)
x2 = ot.Normal(2, 1)
x_funk = ot.JointDistribution([x1, x2], copula)

x1 = ot.Normal(1.0, 1)
x2 = ot.Normal(-2, 1)
x_punk = ot.JointDistribution([x1, x2], copula)
mixture = ot.Mixture([x_funk, x_punk], [0.5, 1.0])

# %%
# The constructed graph is composed of the superposition of a filled contour and iso lines.
# We also changed the thickness and style of the lines to show the effect although it is not useful here.

# %%
graph = mixture.drawPDF([-5.0, -5.0], [5.0, 5.0])
# Add level lines above filled contour
contour = graph.getDrawable(0).getImplementation()
contour.setColor("black")
contour.setColorBarPosition("")
contour.setLineWidth(3)
contour.setLineStyle("dotdash")
graph.add(contour)
# Modify previous contour to fill the graph and use log norm
contour = graph.getDrawable(0).getImplementation()
contour.setIsFilled(True)
contour.setColorMapNorm("log")
graph.setDrawable(0, contour)
view = otv.View(graph)

# %%
# If the color bar is not sufficiently meaningful, it is possible to add the labels of the values of each level line on the drawing.
# Here the labels are reformatted to use scientific notation and define precision.
contour = graph.getDrawable(0).getImplementation()
contour.setColorBarPosition("")  # Hide color bar
graph.setDrawable(0, contour)
contour = graph.getDrawable(1).getImplementation()
contour.setDrawLabels(True)
contour.setLabels(["{:.3g}".format(level) for level in contour.getLevels()])
graph.setDrawable(1, contour)
view = otv.View(graph)

otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()
ot.PlatformInfo.SetNumericalPrecision(6)
ot.ResourceMap.SetAsUnsignedInteger(
    "LinearCombinationDistribution-DefaultMaxSize", 4000000
)


# 2D test using FFT
# Collection of distributions
coll = ot.DistributionCollection(0)
coll.add(ot.Uniform(0, 1))
coll.add(ot.Uniform(0, 1))
coll.add(ot.Uniform(0, 1))
# Set weights
weights = ot.Matrix([[1.0, -2.0, 1.0], [1.0, 1.0, -3.0]])
# Build the LinearCombinationDistribution
dist_2D = ot.LinearCombinationDistribution(coll, weights)
# Defining LinearCombinationDistribution
N = 16
points = ot.Indices(2, N)
mean = dist_2D.getMean()
sigma = dist_2D.getStandardDeviation()
xMin = mean - 2.9 * sigma
xMax = mean + 2.9 * sigma
grid = ot.Sample()
print("distribution = ", repr(dist_2D))
print("range = ", dist_2D.getRange())
print("mean = ", dist_2D.getMean())
print("cov = ", dist_2D.getCovariance())
print("sigma = ", dist_2D.getStandardDeviation())
print("xMin = ", xMin)
print("xMax = ", xMax)
result, grid = dist_2D.computePDF(xMin, xMax, points)
for i in range(grid.getSize()):
    print("%.6g;%.6g;%.6g" % (grid[i][0], grid[i][1], result[i][0]))
# Defining new case, involving Normal distributions
collection = ot.DistributionCollection(0)
collection.add(ot.Normal(2.0, 3.0))
collection.add(ot.Normal(1.0, 4.0))
weights = ot.Matrix([[4.0, 1.0], [2.0, 1.4]])
# Build the LinearCombinationDistribution
distribution2D = ot.LinearCombinationDistribution(collection, weights)
print("distribution = ", repr(distribution2D))
print("range = ", distribution2D.getRange())
print("mean = ", distribution2D.getMean())
print("cov = ", distribution2D.getCovariance())
print("sigma = ", distribution2D.getStandardDeviation())
xMin = distribution2D.getMean() - 2.9 * distribution2D.getStandardDeviation()
xMax = distribution2D.getMean() + 2.9 * distribution2D.getStandardDeviation()
result, grid = distribution2D.computePDF(xMin, xMax, points)
print("x;y;PDF")
for i in range(grid.getSize()):
    print("%.6g;%.6g;%.6g" % (grid[i][0], grid[i][1], result[i][0]))



#! /usr/bin/env python

# We sample the from the posterior distribution of the parameters of a mixture model.
# This example is drawn from Example 9.2 from *Monte-Carlo Statistical methods* by Robert and Casella (2004).

import openturns as ot
import openturns.testing as ott
import numpy as np

ot.TESTPREAMBLE()

ot.RandomGenerator.SetSeed(100)

# Sample data with :math:`\mu_0 = 0` and :math:`\mu_1 = 2.7`.
N = 500
p = 0.3

mu0 = 0.0
mu1 = 2.7
nor0 = ot.Normal(mu0, 1.0)
nor1 = ot.Normal(mu1, 1.0)
true_distribution = ot.Mixture([nor0, nor1], [1 - p, p])
observations = np.array(true_distribution.getSample(500))


def nor0post(pt):
    z = np.array(pt)[2:]
    x0 = observations[z == 0]
    mu0 = x0.sum() / (0.1 + len(x0))
    sigma0 = 1.0 / (0.1 + len(x0))
    return [mu0, sigma0]


def nor1post(pt):
    z = np.array(pt)[2:]
    x1 = observations[z == 1]
    mu1 = x1.sum() / (0.1 + len(x1))
    sigma1 = 1.0 / (0.1 + len(x1))
    return [mu1, sigma1]


def zpost(pt):
    mu0 = pt[0]
    mu1 = pt[1]
    term1 = p * np.exp(-((observations - mu1) ** 2) / 2)
    term0 = (1.0 - p) * np.exp(-((observations - mu0) ** 2) / 2)
    res = term1 / (term1 + term0)
    # output must be a 1d list or array in order to create a PythonFunction
    return res.reshape(-1)


nor0posterior = ot.PythonFunction(2 + N, 2, nor0post)
nor1posterior = ot.PythonFunction(2 + N, 2, nor1post)
zposterior = ot.PythonFunction(2 + N, N, zpost)

# We can now construct the Gibbs algorithm
initialState = [0.0] * (N + 2)
sampler0 = ot.RandomVectorMetropolisHastings(
    ot.RandomVector(ot.Normal()), initialState, [0], nor0posterior
)
sampler1 = ot.RandomVectorMetropolisHastings(
    ot.RandomVector(ot.Normal()), initialState, [1], nor1posterior
)
big_bernoulli = ot.JointDistribution([ot.Bernoulli()] * N)
sampler2 = ot.RandomVectorMetropolisHastings(
    ot.RandomVector(big_bernoulli), initialState, range(2, N + 2), zposterior
)
gibbs = ot.Gibbs([sampler0, sampler1, sampler2])

# Run the Gibbs algorithm
s = gibbs.getSample(1000)

# Extract the relevant marginals: the first (:math:`mu_0`) and the second (:math:`\mu_1`).
posterior_sample = s[:, 0:2]
mean = posterior_sample.computeMean()
stddev = posterior_sample.computeStandardDeviation()
print(mean, stddev)
ott.assert_almost_equal(mean, [-0.078428, 2.80587])
ott.assert_almost_equal(stddev, [0.0463082, 0.108863])



import openturns as ot
import openturns.viewer as otv

lowerBound = [0.0, 0.0]
upperBound = [1.0, 1.0]
interv = ot.Interval(lowerBound, upperBound)
mesher = ot.IntervalMesher([4] * 2)
inputMesh = mesher.build(interv)

lowerBound = [0.0, 0.0]
upperBound = [1.0, 1.0]
interv = ot.Interval(lowerBound, upperBound)
mesher = ot.IntervalMesher([8] * 2)
tempMesh = mesher.build(interv)
transform = ot.SymbolicFunction(["x", "y"], ["x^2", "y^2"])
outputMesh = ot.Mesh(transform(tempMesh.getVertices()), tempMesh.getSimplices())

f = ot.SymbolicFunction(["x", "y"], ["sin(pi_*x)*sin(pi_*y)"])
values_inputMesh = f(inputMesh.getVertices())
input_field = ot.Field(inputMesh, values_inputMesh)

dim_field = 1
func_P1 = ot.P1LagrangeInterpolation(inputMesh, tempMesh, dim_field)

values_outputMesh = func_P1(values_inputMesh)
output_field = ot.Field(tempMesh, values_outputMesh)

graph_1 = input_field.drawMarginal(0)
graph_1.setXTitle("$x$")
graph_1.setYTitle("$y$")
graph_1.setTitle(r"Function $x \rightarrow sin(\pi x)sin(\pi y)$ on the input mesh")
graph_1.setLegendPosition("")

graph_2 = output_field.drawMarginal(0)
graph_2.setLegendPosition("")
graph_2.setXTitle("$x$")
graph_2.setYTitle("$y$")
graph_2.setTitle(r"Function $x \rightarrow sin(\pi x)sin(\pi y)$ on the output mesh")

grid = ot.GridLayout(1, 2)
grid.setGraph(0, 0, graph_1)
grid.setGraph(0, 1, graph_2)


view = otv.View(grid)
view.show()



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


center = ot.Point([0.5, 1.5, 2.5, 3.5])
center.setName("center")
levels = ot.Point([4, 8, 16])
levels.setName("levels")
experiment = ot.Axial(center, levels)
print("experiment = ", experiment)
sample = experiment.generate()
print("sample = ", repr(sample))



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()
ot.RandomGenerator.SetSeed(0)


def compute_max_error(ref_copula, est_copula):
    """
    Compute max error between ref_copula & estimated one
    Error is evaluated in the set (u, v) where both belong to
    {0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0}
    """
    max_error = 0.0
    for n in range(11):
        for p in range(11):
            point = [0.1 * n, 0.1 * p]
            max_error = max(
                max_error,
                abs(ref_copula.computeCDF(point) - est_copula.computeCDF(point)),
            )
    return max_error


def check_bernstein_copula(est_copula):
    """
    Check if an estimated distribution of kind EmpiricalBernstein
    is of type copula and all marginals are also.
    """
    print("Is estimation a copula ? --> ", est_copula.isCopula())
    print("Maginal checking")
    dimension = est_copula.getDimension()
    for d in range(dimension):
        print("Is marginal %d a copula ? --> %s" % (d, est_copula.isCopula()))


coll = [ot.GumbelCopula(3.0), ot.ClaytonCopula(3.0), ot.FrankCopula(3.0)]
size = 100
for i, ref_copula in enumerate(coll):
    ref_copula = coll[i]
    print("Reference copula", str(ref_copula))
    sample = ref_copula.getSample(size)
    # Default method: log-likelihood
    m = ot.BernsteinCopulaFactory.ComputeLogLikelihoodBinNumber(sample)
    print("Log-likelihood m=", m)
    est_copula = ot.BernsteinCopulaFactory().build(sample, m)
    max_error = compute_max_error(ref_copula, est_copula)
    print("Max. error=%.5f" % max_error)
    check_bernstein_copula(est_copula)
    # AMISE method
    m = ot.BernsteinCopulaFactory.ComputeAMISEBinNumber(sample)
    print("AMISE m=", m)
    est_copula = ot.BernsteinCopulaFactory().build(sample, m)
    max_error = compute_max_error(ref_copula, est_copula)
    print("Max. error=%.5f" % max_error)
    check_bernstein_copula(est_copula)
    # Penalized Csiszar divergence method
    f = ot.SymbolicFunction("t", "-log(t)")
    m = ot.BernsteinCopulaFactory.ComputePenalizedCsiszarDivergenceBinNumber(sample, f)
    print("Penalized Csiszar divergence m=", m)
    est_copula = ot.BernsteinCopulaFactory().build(sample, m)
    max_error = compute_max_error(ref_copula, est_copula)
    print("Max. error=%.5f" % max_error)
    check_bernstein_copula(est_copula)
    print("")



"""
Create an event based on a process
==================================
"""

# %%
# This example gives elements to create an event based on a multivariate stochastic process.
# Let :math:`X: \Omega \times \mathcal{D} \rightarrow \mathbb{R}^d` be a stochastic process
# of dimension :math:`d`, where :math:`\mathcal{D} \in \mathbb{R}^n` is discretized on the mesh
# :math:`\mathcal{M}`. We suppose that :math:`\mathcal{M}` contains :math:`N` vertices.
#
# We define the event :math:`\mathcal{E}` as:
#
# .. math::
#    \begin{aligned}
#      \displaystyle \mathcal{E}(X) = \bigcup_{\vect{t}\in \mathcal{M}}\left\{X_{\vect{t}} \in \mathcal{A} \right\}
#    \end{aligned}
#
# where :math:`\mathcal{A}` is a domain of :math:`\mathbb{R}^d`.
#
# A particular domain :math:`\mathcal{A}` is the cartesian product of type:
#
# .. math::
#    \begin{aligned}
#      \mathcal{A} = \prod_{i=1}^d [a_i,b_i]
#    \end{aligned}
#
# In that case, the library defines :math:`\mathcal{A}` by its both extreme points : :math:`\vect{a}` and :math:`\vect{b}`.
#
# In the general case, :math:`\mathcal{A}` is a *Domain* object that is able to
# check if it contains a given point in :math:`\mathbb{R}^d`.
#
# The library creates an *Event* object from the process :math:`X` and the
# domain :math:`\mathcal{A}`. Then, it is possible to get a realization of the event
# :math:`\mathcal{E}`, which is scalar :math:`1_{\mathcal{E}(X)}(\vect{x}_0, \dots, \vect{x}_{N-1})`
# if :math:`(\vect{x}_0, \dots,\vect{x}_{N-1})` is a realization of :math:`X` on
# :math:`\mathcal{M}`.

# %%
import openturns as ot


# %%
# Define a 2-d mesh
indices = [40, 20]
mesher = ot.IntervalMesher(indices)
lowerBound = [0.0, 0.0]
upperBound = [2.0, 1.0]
interval = ot.Interval(lowerBound, upperBound)
mesh = mesher.build(interval)

# %%
# Create the covariance model
amplitude = [1.0, 2.0, 3.0]
scale = [4.0, 5.0]
spatialCorrelation = ot.CorrelationMatrix(3)
spatialCorrelation[0, 1] = 0.8
spatialCorrelation[0, 2] = 0.6
spatialCorrelation[1, 2] = 0.1
covmodel = ot.ExponentialModel(scale, amplitude, spatialCorrelation)

# Create a normal process
process = ot.GaussianProcess(covmodel, mesh)

# %%
# Create a domain A in R^3: [0.8; 1.2]*[1.5; 1.6]*[0.5; 0.7]
lowerBound = [0.8, 1.5, 0.5]
upperBound = [1.2, 1.6, 0.7]
domain = ot.Interval(lowerBound, upperBound)

# Create the event
event = ot.ProcessEvent(process, domain)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

distribution = ot.LogUniform(1.0, 2.5)
size = 10000
sample = distribution.getSample(size)
factory = ot.LogUniformFactory()
estimatedDistribution = factory.build(sample)
print("Distribution          =", distribution)
print("Estimated distribution=", estimatedDistribution)
estimatedDistribution = factory.build()
print("Default distribution=", estimatedDistribution)
estimatedDistribution = factory.build(distribution.getParameter())
print("Distribution from parameters=", estimatedDistribution)
estimatedLogUniform = factory.buildAsLogUniform(sample)
print("LogUniform          =", distribution)
print("Estimated logUniform=", estimatedLogUniform)
estimatedLogUniform = factory.buildAsLogUniform()
print("Default logUniform=", estimatedLogUniform)
estimatedLogUniform = factory.buildAsLogUniform(distribution.getParameter())
print("LogUniform from parameters=", estimatedLogUniform)



import openturns as ot
import openturns.viewer as otv

center = [0.0]
constant = [3.0]
linear = ot.Matrix([[2.0]])
quadratic = ot.SymmetricTensor([[[5.0]]])
f = ot.QuadraticEvaluation(center, constant, linear, quadratic)

graph = f.draw(-5.0, 5.0)
graph.setTitle(r"Quadratic function $x \mapsto 5x^2+2x+3$")
graph.setXTitle(r"$x$")
graph.setYTitle(r"$y$")
otv.View(graph, figure_kw={"figsize": (8, 4)}, add_legend=True)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

regularGrid = ot.RegularGrid(-1.0, 0.1, 21)
regularGridNearestNeighbourAlgorithm = ot.RegularGridNearestNeighbour(regularGrid)
print("regularGridNearestNeighbourAlgorithm=", regularGridNearestNeighbourAlgorithm)

test = ot.Normal().getSample(20)
for i in range(len(test)):
    index = regularGridNearestNeighbourAlgorithm.query(test[i])
    neighbour = ot.Point(
        1, regularGrid.getValue(regularGridNearestNeighbourAlgorithm.query(test[i]))
    )
    print("Nearest neighbour of", test[i], "=", neighbour, "index=", index)

k = 4
for i in range(len(test)):
    indices = regularGridNearestNeighbourAlgorithm.queryK(test[i], k, True)
    print(k, "nearest neighbours of", test[i], "(indices=", indices)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

Tmin = 0.0
deltaT = 0.1
N = 11

# Initialization of the RegularGrid timeGrid1
timeGrid = ot.RegularGrid(Tmin, deltaT, N)

# Functional basis
basisDimension = 10
basis = []
coefficients = ot.DistributionCollection(basisDimension)

for i in range(basisDimension):
    basis.append(ot.SymbolicFunction("x", "sin(" + str(i) + "*x)"))
    coefficients[i] = ot.Normal(0.0, (1.0 + i))

process = ot.FunctionalBasisProcess(ot.JointDistribution(coefficients), ot.Basis(basis))

# Setting the timeGrid
process.setTimeGrid(timeGrid)

# Print of the process
print("process = ", process)

# Initialization of the TimeSeries
timeSerie = process.getRealization()

# Full prints
print("timeSerie = ", timeSerie)

# Some steps further
stepNumber = 4
print("One future=", process.getFuture(stepNumber))
size = 3
print("Some futures=", process.getFuture(stepNumber, size))



"""
Create a FCE for dependent inputs: transformation vs domination
===============================================================
"""

# %%
# In this example, we create a functional chaos expansion for the
# :ref:`Ishigami function<use-case-ishigami>` when the input distribution
# has dependent marginals.
#
# Refer to :ref:`functional_chaos` to learn more about functional chaos expansion.
#
# We provide one input sample and one output sample of the Ishigami function. We build two meta models:
#
# - Meta model 1: we use an isoprobabilistic transformation that maps the input distribution to another one with independent marginals.
# - Meta model 2: we use the domination method: the basis of the projection space is not orthonormal to the input distribution.
#


# %%
# Define the Ishigami model
# -------------------------

# %%
from openturns.usecases import ishigami_function
import openturns as ot
import openturns.viewer as otv

# %%
# We load the Ishigami model. The `IshigamiModel` data class contains the input
# distribution :math:`\mu_{\inputRV}` of the random vector :math:`\vect{X}=(X_1, X_2, X_3)`
# in `im.distribution`.
im = ishigami_function.IshigamiModel()
input_names = im.distribution.getDescription()

# %%
# We want to introduce some dependence betwenn the components. That is why we have to change the
# input distribution stored in the Ishigami model (which have independent components).
# We use a copula
# that links :math:`(X_1,X_2)` with a :class:`~openturns.ClaytonCopula`. The last component
# :math:`X_3` is independent of :math:`(X_1,X_2)`. The final 3d-copula is a
# :class:`~openturns.BlockIndependentCopula` copula.
# We keep the initial marginal distributions.
copula = ot.BlockIndependentCopula([ot.ClaytonCopula(1.5), ot.IndependentCopula(1)])
input_dist = ot.JointDistribution(
    [im.distribution.getMarginal(i) for i in range(im.dim)], copula
)

# %%
# We generate an input sample from the input distribution and
# we compute the output sample from the Ishigami function that is contained
# in `im.model`.
sampleSize = 1000
inputTrain = im.distribution.getSample(sampleSize)
outputTrain = im.model(inputTrain)

# %%
# We display the relationships between the outputs and the inputs.
grid = ot.VisualTest.DrawPairsXY(inputTrain, outputTrain)
view = otv.View(grid, figure_kw={"figsize": (12.0, 4.0)})

# %%
# We draw the histogramm of the output values.
graph = ot.HistogramFactory().build(outputTrain).drawPDF()
graph.setTitle("Ishigami outputs")
graph.setXTitle("y")
view = otv.View(graph)

# %%
# Meta model 1: Transformation method
# -----------------------------------
# As the input distribution has dependent marginals, we can use an
# :ref:`isoprobabilistic transformation <isoprobabilistic_transformation>` that maps the
# input distribution to
# another one with independent marginals. This new distribution is not specified, therefore the
# default adaptive strategy and the default projection stategy are used.
#
# The default adaptive strategy is defined as follows:
#
# - a basis is built as the tensorization of the univariate
#   polynomials family
#   orthonormal to the standard representative of the input marginals distribution. Then
#   this basis is
#   orthonormal to the distribution denoted by :math:`\tilde{\mu}` which is the tensorization
#   of the standard representative of the input marginals distribution;
# - the enumerate function is chosen according to
#   the `FunctionalChaosAlgorithm-QNorm` parameter of the :class:`~openturns.ResourceMap`:
#   if this parameter is equal to 1, then the :class:`~openturns.LinearEnumerateFunction` class is used,
#   otherwise, the :class:`~openturns.HyperbolicAnisotropicEnumerateFunction` class is used. The default
#   value of the key being 0.5, we use the Hyperbolic Anisotropic EnumerateFunction.
# - the first elements of the basis are used to build the approximation space. The number of elements
#   is computed from the total degree (using the enumerate function of the basis) specified as
#   default value
#   in `FunctionalChaosAlgorithm-MaximumTotalDegree` of the :class:`~openturns.ResourceMap`.
#   The default value being 10, the polynomial approximation space is generated by
#   the polynomials of the basis with maximum total degree equal to 10.
#
# The :class:`~openturns.FunctionalChaosAlgorithm` class uses an isoprobabilistic
# transformation :math:`T`
# that maps :math:`\mu_{\inputRV}` into
# :math:`\tilde{\mu}`. We note :math:`\vect{U} = T(\inputRV)`. In this new
# :math:`\vect{U}` -space, the basis built by the adaptive strategy is
# orthonormal to the distribution :math:`\tilde{\mu}` of the new random vector
# :math:`\vect{U}`.
# The meta model of the transformed Ishigami model :math:`\model \circ T^{-1}` is built.
#
# The projection stategy is not specified neither: we use the least-squares strategy  with no model selection
# if the key `FunctionalChaosAlgorithm-Sparse` of the :class:`~openturns.ResourceMap` is *False* and
# with model selection in the other case,  using the selection algorithm specified by the key
# `FunctionalChaosAlgorithm-FittingAlgorithm`. Considering the default values of the keys, we use a
# :ref:`least-squares strategy <least_squares>` with no model selection.
#
# Then, the meta model built in the :math:`\vect{U}` -space is finally composed with the
# isoprobabilistic transformation to get the meta model of the Ishigami function in the initial
# :math:`\vect{X}` -space.
#
# Note that the final meta model is projected on a basis which is not polynomials,
# due to the action of the isoprobabilistic transformation which is not affine.

# %%
# We create the functional chaos algorithm.
chaos_algo = ot.FunctionalChaosAlgorithm(inputTrain, outputTrain, input_dist)
chaos_algo.setUseDomination(False)
chaos_algo.run()

# %%
# We get the result and the resulting meta model.
chaos_result = chaos_algo.getResult()
metamodel = chaos_result.getMetaModel()

# %%
# In order to validate the meta model, we generate a test sample.
n_valid = 1000
inputTest = input_dist.getSample(n_valid)
outputTest = im.model(inputTest)
metamodel_predictions = metamodel(inputTest)

# %%
# We draw the validation graph and we get the :math:`R^2` score: the meta model is of poor quality.
val = ot.MetaModelValidation(outputTest, metamodel_predictions)
r2Score = val.computeR2Score()[0]
print(f"r2Score with Transformation method = {r2Score:.6f}")
graph = val.drawValidation()
graph.setTitle(f"R2={r2Score * 100:.2f}%, use domination = false")
view = otv.View(graph)


# %%
# Meta model 2: Domination method
# -------------------------------
# Now, we want to use the domination method, which means that the basis created by the
# adaptive strategy is used to project the model. This basis is not orthonormal to
# :math:`\mu_{\inputRV}`.
#
# We use the :meth:`~openturns.FunctionalChaosAlgorithm.setUseDomination` method.
# We implement the same steps as before, until the validation graph.
chaos_algo.setUseDomination(True)
chaos_algo.run()

chaos_result = chaos_algo.getResult()
metamodel_dom = chaos_result.getMetaModel()
metamodel_dom_predictions = metamodel_dom(inputTest)
val = ot.MetaModelValidation(outputTest, metamodel_dom_predictions)
r2Score = val.computeR2Score()[0]
print(f"r2Score with Domination method = {r2Score:.6f}")
graph = val.drawValidation()
graph.setTitle(f"R2={r2Score * 100:.2f}%, use domination = true")
view = otv.View(graph)

# %%
# We can see that the meta model obtained with the domination method is largely better than
# the meta model
# obtained with the Transformation method, even though
# the multivariate basis of the approximation space is not orthonormal to the input distribution
# :math:`\mu_{\inputRV}`.
# It can be explained by the fact that the multivariate tensorized basis used by the
# domination method is able to
# capture the tensorized structure of the Ishigami model. This was not the case of the
# Transformation method which uses a basis in the :math:`\vect{X}` -space which is not tensorized,
# due to the action of the isoprobabilist transformation.

# %%
# Display all figures
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

# We create an empty Sample
N = 100
sample = ot.Sample(N, 2)
print("Sample size=", sample.getSize())

# We populate the empty sample with numerous data
for i in range(N):
    point = ot.Point(2)
    point[0] = 1000 + i
    point[1] = 2000 + i
    sample[i] = point
print("sample=", repr(sample))

# We try to split the sample
sample90 = sample.split(90)
sample10 = sample.split(10)
print("sample  =", repr(sample))
print("sample10=", repr(sample10))
print("sample90=", repr(sample90))



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


myFunc = ot.SymbolicFunction(["t", "x"], ["x + t^2"])
tg = ot.RegularGrid(0.0, 0.2, 6)
myTemporalFunc = ot.VertexValueFunction(myFunc, tg)

print("myTemporalFunc=", myTemporalFunc)
# Get the input and output description
print("myTemporalFunc input description=", myTemporalFunc.getInputDescription())
print("myTemporalFunc output description=", myTemporalFunc.getOutputDescription())
# Get the input and output dimension, based on description
print("myTemporalFunc input dimension=", myTemporalFunc.getInputDimension())
print("myTemporalFunc output dimension=", myTemporalFunc.getOutputDimension())
# Create a TimeSeries
data = ot.Sample(tg.getN(), myFunc.getInputDimension() - 1)
for i in range(data.getSize()):
    for j in range(data.getDimension()):
        data[i, j] = i * data.getDimension() + j
ts = ot.TimeSeries(tg, data)
print("input time series=", ts)
print("output time series=", myTemporalFunc(ts))
# Get the number of calls
print("called ", myTemporalFunc.getCallsNumber(), " times")



#! /usr/bin/env python

import openturns as ot
import math as m

ot.TESTPREAMBLE()

sample = ot.Sample(4, 3)
sample[0] = ot.Point((1, 0, 9))
sample[1] = ot.Point((2, 3, 5))
sample[2] = ot.Point((5, 1, 8))
sample[3] = ot.Point((6, 7, 2))

print("sample=", repr(sample))
print("min=", repr(sample.getMin()))
print("max=", repr(sample.getMax()))
print("mean=", repr(sample.computeMean()))
print("covariance=", repr(sample.computeCovariance()))
print("standard deviation=", repr(sample.computeStandardDeviation()))
print("Pearson correlation=", repr(sample.computeLinearCorrelation()))
print("Spearman correlation=", repr(sample.computeSpearmanCorrelation()))
print("Kendall tau=", repr(sample.computeKendallTau()))
print("range per component=", repr(sample.computeRange()))
print("median per component=", repr(sample.computeMedian()))
print("Variance=", repr(sample.computeVariance()))
print("Skewness=", repr(sample.computeSkewness()))
print("Kurtosis=", repr(sample.computeKurtosis()))
for i in range(10):
    print("Centered moment of order ", i, "=", sample.computeCentralMoment(i))
print("Marginal 1=", repr(sample.getMarginal(1)))
indices = ot.Indices(2)
indices[0] = 2
indices[1] = 0
print("Marginal [2, 0]=", repr(sample.getMarginal(indices)))
print("Rank=", repr(sample.rank()))
print("Sort=", repr(sample.sort()))
print("Sort according to component 0=", repr(sample.sortAccordingToAComponent(0)))
prob = 0.25
print(
    "Quantile per component(",
    prob,
    ")=",
    repr(sample.computeQuantilePerComponent(prob)),
)
probs = [0.25, 0.75]
print(
    "Quantile per component(",
    probs,
    ")=",
    repr(sample.computeQuantilePerComponent(probs)),
)
probs = [0.75, 0.25]
print(
    "Quantile per component(",
    probs,
    ")=",
    repr(sample.computeQuantilePerComponent(probs)),
)
pointCDF = ot.Point(sample.getDimension(), 0.25)
print("Empirical CDF(", repr(pointCDF), "=", sample.computeEmpiricalCDF(pointCDF))
dim = 3
R = ot.CorrelationMatrix(dim)
for i in range(1, dim):
    R[i, i - 1] = 0.25
Rtmp = ot.CorrelationMatrix(dim)
for i in range(dim):
    for j in range(i):
        Rtmp[i, j] = 6.0 * m.asin(R[i, j] / 2.0) / m.pi
print("Pearson correlation (exact)=", repr(R))
print("Spearman correlation (exact)=", repr(Rtmp))
sample[0] = ot.Point((1, 0, 9))
sample[1] = ot.Point((2, 3, 9))
sample[2] = ot.Point((5, 1, 9))
sample[3] = ot.Point((2, 2, 9))
print("Rank=", repr(sample.rank()))
print("Rank component 0=", repr(sample.rank(0)))
print("Sort           =", repr(sample.sort()))
sample2 = ot.Sample(sample)
sample2.sortInPlace()
print("Sort (in place)=", repr(sample2))
print(
    "Sort according to component 0           =",
    repr(sample.sortAccordingToAComponent(0)),
)
sample2 = ot.Sample(sample)
sample2.sortAccordingToAComponentInPlace(0)
print("Sort according to component 0 (in place)=", repr(sample2))
print("Spearman correlation=", repr(sample.computeSpearmanCorrelation()))
print("Kendall tau=", repr(sample.computeKendallTau()))
size = 10000
normal = ot.Normal(ot.Point(dim, 0.0), ot.Point(dim, 1.0), R)
print("Normal=", repr(normal))
print("covariance=", repr(normal.getCovariance()))
normalSample = normal.getSample(size)
print("Empirical covariance=", repr(normalSample.computeCovariance()))
RPearson = normalSample.computeLinearCorrelation()
print("Pearson correlation=", repr(RPearson))
RSpearman = normalSample.computeSpearmanCorrelation()
print("Spearman correlation=", repr(RSpearman))
print("Unique           =", repr(sample.sortUnique()))
sample2 = ot.Sample(sample)
sample2.sortUniqueInPlace()
print("Unique (in place)=", repr(sample2))
sample[2] = ot.Point((1, 0, 9))
print("Unique           =", repr(sample.sortUnique()))
sample2 = ot.Sample(sample)
sample2.sortUniqueInPlace()
print("Unique (in place)=", repr(sample2))



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


inputDimension = 3
outputDimension = 2

# Constant term
constant = ot.SymmetricTensor(inputDimension, outputDimension)
constant[0, 0, 0] = 7.0
constant[0, 0, 1] = -7.0
constant[0, 1, 0] = 8.0
constant[0, 1, 1] = -8.0
constant[0, 2, 0] = 9.0
constant[0, 2, 1] = -9.0
constant[1, 0, 0] = 8.0
constant[1, 0, 1] = -8.0
constant[1, 1, 0] = 10.0
constant[1, 1, 1] = -10.0
constant[1, 2, 0] = 11.0
constant[1, 2, 1] = -11.0
constant[2, 0, 0] = 9.0
constant[2, 0, 1] = -9.0
constant[2, 1, 0] = 11.0
constant[2, 1, 1] = -11.0
constant[2, 2, 0] = 12.0
constant[2, 2, 1] = -12.0

myHessian = ot.ConstantHessian(constant)
myHessian.setName("constantHessian")
inPoint = ot.Point(inputDimension)
inPoint[0] = 7.0
inPoint[1] = 8.0
inPoint[2] = 9.0
outTensor = myHessian.hessian(inPoint)
print("myHessian=", repr(myHessian))
print(myHessian.getName(), "( ", repr(inPoint), " ) = ", repr(outTensor))
